#    Back In Time
#    Copyright (C) 2008-2009 Oprea Dan
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os
import os.path

import pygtk
pygtk.require("2.0")
import gtk
import gettext

import config


_=gettext.gettext


def show_question( parent, config, message ):
	dialog = gtk.MessageDialog( parent, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO )
	dialog.set_markup( message )
	dialog.set_title( config.APP_NAME )
	retVal = dialog.run()
	dialog.destroy()
	return retVal

def show_error( parent, config, message ):
	dialog = gtk.MessageDialog( parent, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK )
	dialog.set_markup( message )
	dialog.set_title( config.APP_NAME )
	retVal = dialog.run()
	dialog.destroy()
	return retVal

def text_input_dialog( parent, config, title, default_value = '' ):
	glade = gtk.glade.XML( os.path.join( config.get_app_path(), 'gnome', 'textinputdialog.glade' ), None, 'backintime' )
	dialog = glade.get_widget( 'TextInputDialog' )
	dialog.set_title( title )
	dialog.set_transient_for( parent )

	edit = glade.get_widget( 'edit_text' )
	edit.set_text( default_value )
	edit.grab_focus()
	
	text = None
	if gtk.RESPONSE_OK == dialog.run():
		text = edit.get_text()

	dialog.destroy()
	return text  




#    Back In Time
#    Copyright (C) 2008-2009 Oprea Dan
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import syslog
import os

def openlog():
	name = os.getenv( 'LOGNAME', 'unknown' )
	syslog.openlog( "backintime (%s)" % name )

def closelog():
	syslog.closelog()

def error( msg ):
	print 'ERROR: ' + msg
	syslog.syslog( syslog.LOG_ERR, 'ERROR: ' + msg )

def warning( msg ):
	print 'WARNING: ' + msg
	syslog.syslog( syslog.LOG_WARNING, 'WARNING: ' + msg )

def info( msg ):
	print 'INFO: ' + msg
	syslog.syslog( syslog.LOG_INFO, 'INFO: ' + msg )




#    Back In Time
#    Copyright (C) 2008-2009 Oprea Dan
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os
import pluginmanager
import tools
import logger
import threading
import time


if len( os.getenv( 'DISPLAY', '' ) ) == 0:
	os.putenv( 'DISPLAY', ':0.0' )


class GnomePlugin( pluginmanager.Plugin ):

	class Systray( threading.Thread ):
		def __init__( self, snapshots ):
			import pygtk
			pygtk.require("2.0")
			import gtk
			import gnome

			threading.Thread.__init__( self )
			self.stop_flag = False
			self.snapshots = snapshots
			self.config = self.snapshots.config

			gnome_props = { gnome.PARAM_APP_DATADIR : '/usr/share' }
			gnome.program_init( 'backintime', self.config.VERSION, properties = gnome_props )

		def start( self ):
			self.stop_flag = False
			threading.Thread.start( self )

		def stop( self ):
			self.stop_flag = True
			try:
				self.join()
			except:
				pass

		def show_notification( self, icon ):
			import pynotify
			self.notification.set_timeout( pynotify.EXPIRES_NEVER )
			self.notification.show()

		def run( self ):
			import gtk
			import pynotify

			logger.info( '[GnomePlugin.Systray.run]' )

			gtk.gdk.threads_init()
			display = gtk.gdk.display_get_default()
			
			if display is None:
				logger.info( '[GnomePlugin.Systray.run] display KO' )
				return

			status_icon = None
			try:
				status_icon = gtk.StatusIcon()
			except:
				pass

			if status_icon is None:
				logger.info( '[GnomePlugin.Systray.run] no status_icon' )
				return

			attach_notification = True
			last_message = None
			first_error = self.config.is_notify_enabled()

			status_icon.set_from_stock( gtk.STOCK_SAVE )
			status_icon.set_visible( True )

			pynotify.init( self.config.APP_NAME )
			self.notification = pynotify.Notification( self.config.APP_NAME, '', '' )
			self.notification.set_urgency( pynotify.URGENCY_NORMAL )
			self.notification.set_timeout( pynotify.EXPIRES_NEVER )

			status_icon.connect('activate', self.show_notification )

			logger.info( '[GnomePlugin.Systray.run] begin loop' )

			while True:
				gtk.main_iteration( False )
				if self.stop_flag:
					break

				if not gtk.events_pending():
					if attach_notification:
						attach_notification = False
						self.notification.attach_to_status_icon( status_icon )

					message = self.snapshots.get_take_snapshot_message()
					if message is None and last_message is None:
						message = ( 0, _('Working...') )

					if not message is None:
						if message != last_message:
							last_message = message

							urgency = pynotify.URGENCY_NORMAL
							icon_name = gtk.STOCK_SAVE
							status_icon_blinking = False
							if last_message[0] != 0:
								urgency = pynotify.URGENCY_CRITICAL
								icon_name = 'dialog-error'
								status_icon_blinking = True

							status_icon.set_blinking( status_icon_blinking )
							status_icon.set_tooltip( last_message[1] )

							self.notification.update( self.config.APP_NAME, last_message[1], icon_name )
							self.notification.set_urgency( urgency )
						
							if last_message[0] != 0 and first_error:
								first_error = False
								self.notification.set_timeout( 10000 )
								self.notification.show()

					time.sleep( 0.2 )
			
			status_icon.set_visible( False )
			gtk.main_iteration( False )

			self.notification.close()
			pynotify.uninit()
			
			logger.info( '[GnomePlugin.Systray.run] end loop' )

	def __init__( self ):
		return

	def init( self, snapshots ):
		if not tools.process_exists( 'gnome-settings-daemon' ):
			return False

		if not tools.check_x_server():
			return False

		self.systray = None
		try:
			self.systray = GnomePlugin.Systray( snapshots )
		except:
			self.systray = None

		return True

	def is_gui( self ):
		return True

	def on_process_begins( self ):
		if not self.systray is None:
			self.systray.start()

	def on_process_ends( self ):
		if not self.systray is None:
			self.systray.stop()

	def on_error( self, code, message ):
		return

	def on_new_snapshot( self, snapshot_id, snapshot_path ):
		return




#    Back In Time
#    Copyright (C) 2008-2009 Oprea Dan
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os.path
import os
import datetime
import gettext

import configfile
import tools


_=gettext.gettext

gettext.bindtextdomain( 'backintime', '/usr/share/locale' )
gettext.textdomain( 'backintime' )


class Config( configfile.ConfigFile ):
	APP_NAME = 'Back In Time'
	VERSION = '0.9.26'
	COPYRIGHT = 'Copyright (c) 2008-2009 Oprea Dan'
	CONFIG_VERSION = 2

	NONE = 0
	_5_MIN = 2
	_10_MIN = 4
	HOUR = 10
	DAY = 20
	WEEK = 30
	MONTH = 40
	YEAR = 80

	DISK_UNIT_MB = 10
	DISK_UNIT_GB = 20

	AUTOMATIC_BACKUP_MODES = { 
				NONE : _('Disabled'), 
				_5_MIN: _('Every 5 minutes'), 
				_10_MIN: _('Every 10 minutes'), 
				HOUR : _('Every Hour'), 
				DAY : _('Every Day'), 
				WEEK : _('Every Week'), 
				MONTH : _('Every Month')
				}

	REMOVE_OLD_BACKUP_UNITS = { 
				DAY : _('Day(s)'), 
				WEEK : _('Week(s)'), 
				YEAR : _('Year(s)')
				}

	MIN_FREE_SPACE_UNITS = { DISK_UNIT_MB : 'Mb', DISK_UNIT_GB : 'Gb' }


	def __init__( self ):
		self._APP_PATH =  os.path.dirname( os.path.abspath( os.path.dirname( __file__ ) ) )
		self._DOC_PATH = '/usr/share/doc/backintime'
		if os.path.exists( os.path.join( self._APP_PATH, 'LICENSE' ) ):
			self._DOC_PATH = self._APP_PATH

		self._GLOBAL_CONFIG_PATH = '/etc/backintime/config'

		HOME_FOLDER = os.path.expanduser( '~' )
		self._LOCAL_DATA_FOLDER = os.path.join( os.getenv( 'XDG_DATA_HOME', '$HOME/.local/share' ).replace( '$HOME', HOME_FOLDER ), 'backintime' )
		self._LOCAL_CONFIG_FOLDER = os.path.join( os.getenv( 'XDG_CONFIG_HOME', '$HOME/.config' ).replace( '$HOME', HOME_FOLDER ), 'backintime' )

		#self._LOCAL_CONFIG_FOLDER = os.path.expanduser( '~/.config/backintime' )
		tools.make_dirs( self._LOCAL_CONFIG_FOLDER )
		tools.make_dirs( self._LOCAL_DATA_FOLDER )

		self._LOCAL_CONFIG_PATH = os.path.join( self._LOCAL_CONFIG_FOLDER, 'config' )
		old_path = os.path.join( self._LOCAL_CONFIG_FOLDER, 'config2' )

		if os.path.exists( old_path ):
			if os.path.exists( self._LOCAL_CONFIG_PATH ):
				os.system( "rm \"%s\"" % old_path )
			else:
				os.system( "mv \"%s\" \"%s\"" % ( old_path, self._LOCAL_CONFIG_PATH ) )

		self.load( self._GLOBAL_CONFIG_PATH )
		self.append( self._LOCAL_CONFIG_PATH )

		if self.get_int_value( 'config.version', 1 ) < 2:
			#remap old items
			self.remap_key( 'BASE_BACKUP_PATH', 'snapshots.path' )
			self.remap_key( 'INCLUDE_FOLDERS', 'snapshots.include_folders' )
			self.remap_key( 'EXCLUDE_PATTERNS', 'snapshots.exclude_patterns' )
			self.remap_key( 'AUTOMATIC_BACKUP', 'snapshots.automatic_backup_mode' )
			self.remap_key( 'REMOVE_OLD_BACKUPS', 'snapshots.remove_old_snapshots.enabled' )
			self.remap_key( 'REMOVE_OLD_BACKUPS_VALUE', 'snapshots.remove_old_snapshots.value' )
			self.remap_key( 'REMOVE_OLD_BACKUPS_UNIT', 'snapshots.remove_old_snapshots.unit' )
			self.remap_key( 'MIN_FREE_SPACE', 'snapshots.min_free_space.enabled' )
			self.remap_key( 'MIN_FREE_SPACE_VALUE', 'snapshots.min_free_space.value' )
			self.remap_key( 'MIN_FREE_SPACE_UNIT', 'snapshots.min_free_space.unit' )
			self.remap_key( 'DONT_REMOVE_NAMED_SNAPSHOTS', 'snapshots.dont_remove_named_snapshots' )
			self.remap_key( 'DIFF_CMD', 'gnome.diff.cmd' )
			self.remap_key( 'DIFF_CMD_PARAMS', 'gnome.diff.params' )
			self.remap_key( 'LAST_PATH', 'gnome.last_path' )
			self.remap_key( 'MAIN_WINDOW_X', 'gnome.main_window.x' )
			self.remap_key( 'MAIN_WINDOW_Y', 'gnome.main_window.y' )
			self.remap_key( 'MAIN_WINDOW_WIDTH', 'gnome.main_window.width' )
			self.remap_key( 'MAIN_WINDOW_HEIGHT', 'gnome.main_window.height' )
			self.remap_key( 'MAIN_WINDOW_HPANED1_POSITION', 'gnome.main_window.hpaned1' )
			self.remap_key( 'MAIN_WINDOW_HPANED2_POSITION', 'gnome.main_window.hpaned2' )

		if self.get_int_value( 'config.version', 1 ) < self.CONFIG_VERSION:
			self.set_int_value( 'config.version', self.CONFIG_VERSION )

	def save( self ):
		configfile.ConfigFile.save( self, self._LOCAL_CONFIG_PATH )

	def check_take_snapshot_params( self, snapshots_path, include_list, exclude_list ):
		#returns None or ( ID, message ) //0 - snapshots path, 1 - include list, 2 - exclude list
		if len( snapshots_path ) == 0 or not os.path.isdir( snapshots_path ):
			return ( 0, _('Snapshots folder is not valid !') )

		if len( snapshots_path ) <= 1:
			return ( 0, _('Snapshots folder can\'t be the root folder !') )

		if len( include_list ) <= 0:
			return ( 1, _('You must select at least one folder to backup !') )

		snapshots_path2 = snapshots_path + '/'

		for item in include_list:
			path = item[0]
			if path == snapshots_path:
				print "Path: " + path
				print "SnapshotsPath: " + snapshots_path 
				return ( 1, _('You can\'t include backup folder !') )
			
			if len( path ) >= len( snapshots_path2 ):
				if path[ : len( snapshots_path2 ) ] == snapshots_path2:
					print "Path: " + path
					print "SnapshotsPath2: " + snapshots_path2
					return ( 1, _('You can\'t include a backup sub-folder !') )
			#else:
			#	path2 = path + '/'
			#	if len( path2 ) < len( snapshots_path ):
			#		if path2 == snapshots_path[ : len( path2 ) ]:
			#			print "Path2: " + path2
			#			print "SnapshotsPath: " + snapshots_path 
			#			return ( 1, _('"Backup folders" can\'t include snapshots folder !') )

		for exclude in exclude_list:
			if exclude.find( ':' ) >= 0:
				return ( 2, _('Exclude patterns can\'t contain \':\' char !') )

		return None

	def get_snapshots_path( self ):
		return self.get_str_value( 'snapshots.path', '' )

	def get_snapshots_full_path( self ):
		return os.path.join( self.get_snapshots_path(), 'backintime' ) 

	def set_snapshots_path( self, value ):
		print "Snapshots path: %s" % value

		if len( value ) > 0:
			if not os.path.isdir( value ):
				return _( '%s is not a folder !' ) % value
			else:
				old_value = self.get_snapshots_path()
				self.set_str_value( 'snapshots.path', value )
				full_path = self.get_snapshots_full_path()
				self.set_str_value( 'snapshots.path', old_value )

				if not os.path.isdir( full_path ):
					tools.make_dirs( full_path )
					if not os.path.isdir( full_path ):
						return _( 'Can\'t write to: %s\nAre you sure have write access ?' ) % value

		self.set_str_value( 'snapshots.path', value )
		return None

	def get_include_folders( self ):
		value = self.get_str_value( 'snapshots.include_folders', '' )
		if len( value ) <= 0:
			return []

		paths = []

		for item in value.split(':'):
			fields = item.split( '|' )

			path = os.path.expanduser( fields[0] )
			path = os.path.abspath( path )

			if len( fields ) >= 2:
				automatic_backup_mode = int( fields[1] )
			else:
				automatic_backup_mode = self.get_automatic_backup_mode()

			paths.append( ( path, automatic_backup_mode ) )

		return paths

	def set_include_folders( self, list ):
		value = ''

		for item in list:
			if len( value ) > 0:
				value = value + ':'
			value = value + item[0] + '|' + str( item[1] )

		self.set_str_value( 'snapshots.include_folders', value )

	def get_exclude_patterns( self ):
		value = self.get_str_value( 'snapshots.exclude_patterns', '.*:*.backup*:*~' )
		if len( value ) <= 0:
			return []
		return value.split(':')

	def set_exclude_patterns( self, list ):
		self.set_str_value( 'snapshots.exclude_patterns', ':'.join( list ) )

	def get_automatic_backup_mode( self ):
		return self.get_int_value( 'snapshots.automatic_backup_mode', self.NONE )

	def set_automatic_backup_mode( self, value ):
		self.set_int_value( 'snapshots.automatic_backup_mode', value )
		#self.setup_cron()

	def get_per_directory_schedule( self ):
		return self.get_bool_value( 'snapshots.expert.per_directory_schedule', False )

	def set_per_directory_schedule( self, value ):
		return self.set_bool_value( 'snapshots.expert.per_directory_schedule', value )

	def get_remove_old_snapshots( self ):
		return ( self.get_bool_value( 'snapshots.remove_old_snapshots.enabled', True ),
				 self.get_int_value( 'snapshots.remove_old_snapshots.value', 10 ),
				 self.get_int_value( 'snapshots.remove_old_snapshots.unit', self.YEAR ) )
	
	def is_remove_old_snapshots_enabled( self ):
		return self.get_bool_value( 'snapshots.remove_old_snapshots.enabled', True )
	
	def get_remove_old_snapshots_date( self ):
		enabled, value, unit = self.get_remove_old_snapshots()
		if not enabled:
			return datetime.date( 1, 1, 1 )

		if unit == self.DAY: 
			date = datetime.date.today()
			date = date - datetime.timedelta( days = value )
			return date

		if unit == self.WEEK:
			date = datetime.date.today()
			date = date - datetime.timedelta( days = date.weekday() + 7 * value )
			return date
		
		if unit == self.YEAR:
			date = datetime.date.today()
			return date.replace( year = date.year - value )
		
		return datetime.date( 1, 1, 1 )

	def set_remove_old_snapshots( self, enabled, value, unit ):
		self.set_bool_value( 'snapshots.remove_old_snapshots.enabled', enabled )
		self.set_int_value( 'snapshots.remove_old_snapshots.value', value )
		self.set_int_value( 'snapshots.remove_old_snapshots.unit', unit )

	def get_min_free_space( self ):
		return ( self.get_bool_value( 'snapshots.min_free_space.enabled', True ),
				 self.get_int_value( 'snapshots.min_free_space.value', 1 ),
				 self.get_int_value( 'snapshots.min_free_space.unit', self.DISK_UNIT_GB ) )
	
	def is_min_free_space_enabled( self ):
		return self.get_bool_value( 'snapshots.min_free_space.enabled', True )

	def get_min_free_space_in_mb( self ):
		enabled, value, unit = self.get_min_free_space()
		if not enabled:
			return 0

		if self.DISK_UNIT_MB == unit:
			return value

		value *= 1024 #Gb
		if self.DISK_UNIT_GB == unit:
			return value

		return 0

	def set_min_free_space( self, enabled, value, unit ):
		self.set_bool_value( 'snapshots.min_free_space.enabled', enabled )
		self.set_int_value( 'snapshots.min_free_space.value', value )
		self.set_int_value( 'snapshots.min_free_space.unit', unit )

	def get_dont_remove_named_snapshots( self ):
		return self.get_bool_value( 'snapshots.dont_remove_named_snapshots', True )

	def set_dont_remove_named_snapshots( self, value ):
		self.set_bool_value( 'snapshots.dont_remove_named_snapshots', value )
	
	def get_smart_remove( self ):
		return self.get_bool_value( 'snapshots.smart_remove', False )

	def set_smart_remove( self, value ):
		self.set_bool_value( 'snapshots.smart_remove', value )
	
	def is_notify_enabled( self ):
		return self.get_bool_value( 'snapshots.notify.enabled', True )

	def set_notify_enabled( self, value ):
		self.set_bool_value( 'snapshots.notify.enabled', value )

	def is_run_nice_from_cron_enabled( self ):
		return self.get_bool_value( 'snapshots.cron.nice', True )

	def set_run_nice_from_cron_enabled( self, value ):
		self.set_bool_value( 'snapshots.cron.nice', value )

	def get_take_snapshot_user_script( self, step ):
		return self.get_str_value( "snapshots.take_snapshot.%s.user.script" )

	def set_take_snapshot_user_script( self, step, path ):
		self.set_str_value( "snapshots.take_snapshot.%s.user.script" % step, path )

	def get_take_snapshot_user_script_before( self ):
		return self.get_take_snapshot_user_script( 'before' )

	def set_take_snapshot_user_script_before( self, path ):
		self.set_take_snapshot_user_script( 'before', path )

	def get_take_snapshot_user_script_after( self ):
		return self.get_take_snapshot_user_script( 'after' )

	def set_take_snapshot_user_script_after( self, path ):
		self.set_take_snapshot_user_script( 'after', path )

	def get_take_snapshot_user_script_new_snapshot( self ):
		return self.get_take_snapshot_user_script( 'new_snapshot' )

	def set_take_snapshot_user_script_new_snapshot( self, path ):
		self.set_take_snapshot_user_script( 'new_snapshot', path )

	def get_take_snapshot_user_script_error( self ):
		return self.get_take_snapshot_user_script( 'error' )

	def set_take_snapshot_user_script_error( self, path ):
		self.set_take_snapshot_user_script( 'error', path )

	def get_app_path( self ):
		return self._APP_PATH

	def get_doc_path( self ):
		return self._DOC_PATH

	def get_app_instance_file( self ):
		return os.path.join( self._LOCAL_DATA_FOLDER, 'app.lock' )

	def get_take_snapshot_message_file( self ):
		return os.path.join( self._LOCAL_DATA_FOLDER, 'worker.message' )

	def get_take_snapshot_instance_file( self ):
		return os.path.join( self._LOCAL_DATA_FOLDER, 'worker.lock' )

	def get_last_snapshot_info_file( self ):
		return os.path.join( self._LOCAL_DATA_FOLDER, 'snapshot.last' )

	def get_take_snapshot_user_callback( self ):
		return os.path.join( self._LOCAL_CONFIG_FOLDER, 'user.callback' )

	def get_license( self ):
		return tools.read_file( os.path.join( self.get_doc_path(), 'LICENSE' ), '' )

	def get_translations( self ):
		return tools.read_file( os.path.join( self.get_doc_path(), 'TRANSLATIONS' ), '' )

	def get_authors( self ):
		return tools.read_file( os.path.join( self.get_doc_path(), 'AUTHORS' ), '' )

	def prepare_path( self, path ):
		if len( path ) > 1:
			if path[ -1 ] == os.sep:
				path = path[ : -1 ]
		return path

	def is_configured( self ):
		if len( self.get_snapshots_path() ) == 0:
			return False

		if len( self.get_include_folders() ) == 0:
			return False

		return True

	def can_backup( self ):
		if not self.is_configured():
			return False

		if not os.path.isdir( self.get_snapshots_full_path() ):
			return False

		return True

	def setup_cron( self ):
		#auto_backup_mode = self.get_automatic_backup_mode()
		min_backup_mode = self.NONE
		max_backup_mode = self.NONE

		if self.get_per_directory_schedule():
			for item in self.get_include_folders():
				backup_mode = item[1]

				if self.NONE != backup_mode:
					if self.NONE == min_backup_mode:
						min_backup_mode = backup_mode
						max_backup_mode = backup_mode
					elif backup_mode < min_backup_mode:
						min_backup_mode = backup_mode
					elif backup_mode > max_backup_mode:
						max_backup_mode = backup_mode
		
			print "Min automatic backup: %s" % self.AUTOMATIC_BACKUP_MODES[ min_backup_mode ]
			print "Max automatic backup: %s" % self.AUTOMATIC_BACKUP_MODES[ max_backup_mode ]
		else:
			min_backup_mode = self.get_automatic_backup_mode()
			max_backup_mode = min_backup_mode
			print "Automatic backup: %s" % self.AUTOMATIC_BACKUP_MODES[ min_backup_mode ]

		if self.NONE == min_backup_mode:
			os.system( "crontab -l | grep -v backintime | crontab -" )
			return None #no automatic backup

		if not tools.check_command( 'crontab' ):
			return _( 'Can\'t find crontab.\nAre you sure cron is installed ?\nIf not you should disable all automatic backups.' )

		#remove old cron
		os.system( "crontab -l | grep -v backintime | crontab -" )

		cron_line = ''
		
		if self._5_MIN == min_backup_mode:
			cron_line = 'echo "*/5 * * * * {cmd}"'
		elif self._10_MIN == min_backup_mode:
			cron_line = 'echo "*/10 * * * * {cmd}"'
		if self.HOUR == min_backup_mode:
			cron_line = 'echo "@hourly {cmd}"'
		elif self.DAY == min_backup_mode:
			cron_line = 'echo "@daily {cmd}"'
		elif self.WEEK == min_backup_mode and self.MONTH == max_backup_mode: #for every-week and every-month use every-day
			cron_line = 'echo "@daily {cmd}"'
		elif self.WEEK == min_backup_mode:
			cron_line = 'echo "@weekly {cmd}"'
		elif self.MONTH == min_backup_mode:
			cron_line = 'echo "@monthly {cmd}"'

		if len( cron_line ) > 0:
			cmd = '/usr/bin/backintime --backup-job >/dev/null 2>&1'
			if self.is_run_nice_from_cron_enabled():
				cmd = 'nice -n 19 ' + cmd
			cron_line = cron_line.replace( '{cmd}', cmd )
			os.system( "( crontab -l; %s ) | crontab -" % cron_line )

		return None


if __name__ == "__main__":
	config = Config()
	print "snapshots path = %s" % config.get_snapshots_full_path()




#    Back In Time
#    Copyright (C) 2008-2009 Oprea Dan
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os
import os.path
import sys
import pygtk
pygtk.require("2.0")
import gtk
import gnomevfs
import gobject
import gtk.glade
import datetime
import gettext

import config
import tools
import clipboardtools 
import messagebox
import gnometools


_=gettext.gettext


class SnapshotsDialog:
	def __init__( self, snapshots, parent, path, snapshots_list, current_snapshot_id, icon_name ):
		self.snapshots = snapshots
		self.config = snapshots.config

		self.glade = gtk.glade.XML( os.path.join( self.config.get_app_path(), 'gnome', 'snapshotsdialog.glade' ), None, 'backintime' )

		self.path = None
		self.icon_name = None

		self.dialog = self.glade.get_widget( 'SnapshotsDialog' )
		self.dialog.set_transient_for( parent )

		signals = { 
			'on_list_snapshots_cursor_changed' : self.on_list_snapshots_cursor_changed,
			'on_list_snapshots_row_activated' : self.on_list_snapshots_row_activated,
			'on_list_snapshots_popup_menu' : self.on_list_snapshots_popup_menu,
			'on_list_snapshots_button_press_event': self.on_list_snapshots_button_press_event,
			'on_list_snapshots_drag_data_get': self.on_list_snapshots_drag_data_get,
			'on_btn_diff_with_clicked' : self.on_btn_diff_with_clicked,
			'on_btn_copy_snapshot_clicked' : self.on_btn_copy_snapshot_clicked,
			'on_btn_restore_snapshot_clicked' : self.on_btn_restore_snapshot_clicked
			}

		#path
		self.edit_path = self.glade.get_widget( 'edit_path' )

		#diff
		self.edit_diff_cmd = self.glade.get_widget( 'edit_diff_cmd' )
		self.edit_diff_cmd_params = self.glade.get_widget( 'edit_diff_cmd_params' )

		diff_cmd = self.config.get_str_value( 'gnome.diff.cmd', 'meld' )
		diff_cmd_params = self.config.get_str_value( 'gnome.diff.params', '%1 %2' )

		self.edit_diff_cmd.set_text( diff_cmd )
		self.edit_diff_cmd_params.set_text( diff_cmd_params )

		#setup backup folders
		self.list_snapshots = self.glade.get_widget( 'list_snapshots' )
		self.list_snapshots.drag_source_set( gtk.gdk.BUTTON1_MASK | gtk.gdk.BUTTON3_MASK, gtk.target_list_add_uri_targets(), gtk.gdk.ACTION_COPY )

		self.glade.signal_autoconnect( signals )
		
		text_renderer = gtk.CellRendererText()
		column = gtk.TreeViewColumn( _('Snapshots') )
		column.pack_end( text_renderer, True )
		column.add_attribute( text_renderer, 'markup', 0 )
		column.set_sort_column_id( 0 )
		self.list_snapshots.append_column( column )

		#display name, snapshot_id
		self.store_snapshots = gtk.ListStore( str, str )
		self.list_snapshots.set_model( self.store_snapshots )

		self.store_snapshots.set_sort_column_id( 0, gtk.SORT_DESCENDING )

		#setup diff with combo
		self.combo_diff_with = self.glade.get_widget( 'combo_diff_with' )
		text_renderer = gtk.CellRendererText()
		self.combo_diff_with.pack_start( text_renderer, True )
		self.combo_diff_with.add_attribute( text_renderer, 'markup', 0 )
		self.combo_diff_with.set_model( self.store_snapshots ) #use the same store

		#UPDATE
		self.path = path
		self.icon_name = icon_name
		self.update_snapshots( current_snapshot_id, snapshots_list )

	def update_toolbar( self ):
		if len( self.store_snapshots ) <= 0:
			self.glade.get_widget( 'btn_copy_snapshot' ).set_sensitive( False )
			self.glade.get_widget( 'btn_restore_snapshot' ).set_sensitive( False )
		else:
			self.glade.get_widget( 'btn_copy_snapshot' ).set_sensitive( True )

			iter = self.list_snapshots.get_selection().get_selected()[1]
			if iter is None:
				self.glade.get_widget( 'btn_restore_snapshot' ).set_sensitive( False )
			else:
				path = self.store_snapshots.get_value( iter, 1 )
				self.glade.get_widget( 'btn_restore_snapshot' ).set_sensitive( len( path ) > 1 )

	def on_btn_restore_snapshot_clicked( self, button ):
		iter = self.list_snapshots.get_selection().get_selected()[1]
		if not iter is None:
			button.set_sensitive( False )
			gnometools.run_gtk_update_loop()
			self.snapshots.restore( self.store_snapshots.get_value( iter, 1 ), self.path )
			button.set_sensitive( True )

	def on_btn_copy_snapshot_clicked( self, button ):
		iter = self.list_snapshots.get_selection().get_selected()[1]
		if not iter is None:
			path = self.snapshots.get_snapshot_path_to( self.store_snapshots.get_value( iter, 1 ), self.path )
			clipboardtools.clipboard_copy_path( path )
 
	def on_list_snapshots_drag_data_get( self, widget, drag_context, selection_data, info, timestamp, user_param1 = None ):
		iter = self.list_snapshots.get_selection().get_selected()[1]
		if not iter is None:
			path = self.store_snapshots.get_value( iter, 2 )
			path = gnomevfs.escape_path_string(path)
			selection_data.set_uris( [ 'file://' + path ] )

	def on_list_snapshots_cursor_changed( self, list ):
		self.update_toolbar()

	def on_list_snapshots_button_press_event( self, list, event ):
		if event.button != 3:
			return

		if len( self.store_snapshots ) <= 0:
			return

		path = self.list_snapshots.get_path_at_pos( int( event.x ), int( event.y ) )
		if path is None:
			return
		path = path[0]
	
		self.list_snapshots.get_selection().select_path( path )
		self.update_toolbar()
		self.show_popup_menu( self.list_snapshots, event.button, event.time )

	def on_list_snapshots_popup_menu( self, list ):
		self.showPopupMenu( list, 1, gtk.get_current_event_time() )

	def show_popup_menu( self, list, button, time ):
		iter = list.get_selection().get_selected()[1]
		if iter is None:
			return

		#print "popup-menu"
		self.popup_menu = gtk.Menu()

		menu_item = gtk.ImageMenuItem( 'backintime.open' )
		menu_item.set_image( gtk.image_new_from_icon_name( self.icon_name, gtk.ICON_SIZE_MENU ) )
		menu_item.connect( 'activate', self.on_list_snapshots_open_item )
		self.popup_menu.append( menu_item )

		self.popup_menu.append( gtk.SeparatorMenuItem() )

		menu_item = gtk.ImageMenuItem( 'backintime.copy' )
		menu_item.set_image( gtk.image_new_from_stock( gtk.STOCK_COPY, gtk.ICON_SIZE_MENU ) )
		menu_item.connect( 'activate', self.on_list_snapshots_copy_item )
		self.popup_menu.append( menu_item )

		menu_item = gtk.ImageMenuItem( gtk.STOCK_JUMP_TO )
		menu_item.connect( 'activate', self.on_list_snapshots_jumpto_item )
		self.popup_menu.append( menu_item )

		path = self.store_snapshots.get_value( iter, 1 )
		if len( path ) > 1:
			menu_item = gtk.ImageMenuItem( 'backintime.restore' )
			menu_item.set_image( gtk.image_new_from_stock( gtk.STOCK_UNDELETE, gtk.ICON_SIZE_MENU ) )
			menu_item.connect( 'activate', self.on_list_snapshots_restore_item )
			self.popup_menu.append( menu_item )

		self.popup_menu.append( gtk.SeparatorMenuItem() )

		menu_item = gtk.ImageMenuItem( 'backintime.diff' )
		#menu_item.set_image( gtk.image_new_from_stock( gtk.STOCK_COPY, gtk.ICON_SIZE_MENU ) )
		menu_item.connect( 'activate', self.on_list_snapshots_diff_item )
		self.popup_menu.append( menu_item )

		self.popup_menu.show_all()
		self.popup_menu.popup( None, None, None, button, time )

	def on_list_snapshots_diff_item( self, widget, data = None ):
		self.on_btn_diff_with_clicked( self.glade.get_widget( 'btn_diff_with' ) )

	def on_list_snapshots_jumpto_item( self, widget, data = None ):
		self.dialog.response( gtk.RESPONSE_OK )

	def on_list_snapshots_open_item( self, widget, data = None ):
		self.open_item()

	def on_list_snapshots_restore_item( self, widget, data = None ):
		self.on_btn_restore_snapshot_clicked( self.glade.get_widget( 'btn_restore_snapshot' ) )

	def on_list_snapshots_copy_item( self, widget, data = None ):
		self.on_btn_copy_snapshot_clicked( self.glade.get_widget( 'btn_copy_snapshot' ) )

	def on_btn_diff_with_clicked( self, button ):
		if len( self.store_snapshots ) < 1:
			return

		#get path from the list
		iter = self.list_snapshots.get_selection().get_selected()[1]
		if iter is None:
			return
		path1 = self.snapshots.get_snapshot_path_to( self.store_snapshots.get_value( iter, 1 ), self.path )

		#get path from the combo
		path2 = self.snapshots.get_snapshot_path_to( self.store_snapshots.get_value( self.combo_diff_with.get_active_iter(), 1 ), self.path )

		#check if the 2 paths are different
		if path1 == path2:
			messagebox.show_error( self.dialog, self.config, _('You can\'t compare a snapshot to itself') )
			return

		diff_cmd = self.edit_diff_cmd.get_text()
		diff_cmd_params = self.edit_diff_cmd_params.get_text()

		if not tools.check_command( diff_cmd ):
			messagebox.show_error( self.dialog, self.config, _('Command not found: %s') % diff_cmd )
			return

		params = diff_cmd_params
		params = params.replace( '%1', "\"%s\"" % path1 )
		params = params.replace( '%2', "\"%s\"" % path2 )

		cmd = diff_cmd + ' ' + params + ' &'
		os.system( cmd  )

		#check if the command changed
		old_diff_cmd = self.config.get_str_value( 'gnome.diff.cmd', 'meld' )
		old_diff_cmd_params = self.config.get_str_value( 'gnome.diff.params', '%1 %2' )
		if diff_cmd != old_diff_cmd or diff_cmd_params != old_diff_cmd_params:
			self.config.set_str_value( 'gnome.diff.cmd', diff_cmd )
			self.config.set_str_value( 'gnome.diff.params', diff_cmd_params )
			self.config.save()

	def update_snapshots( self, current_snapshot_id, snapshots_list ):
		self.edit_path.set_text( self.path )

		#fill snapshots
		self.store_snapshots.clear()
	
		path = self.snapshots.get_snapshot_path_to( current_snapshot_id, self.path )	
		isdir = os.path.isdir( path )

		counter = 0
		index_combo_diff_with = 0
		
		#add now
		path = self.path
		if os.path.exists( path ):
			if os.path.isdir( path ) == isdir:
				self.store_snapshots.append( [ gnometools.get_snapshot_display_markup( self.snapshots, '/' ), '/' ] )
				if '/' == current_snapshot_id:
					indexComboDiffWith = counter
				counter += 1
				
		#add snapshots
		for snapshot in snapshots_list:
			path = self.snapshots.get_snapshot_path_to( snapshot, self.path )
			if os.path.exists( path ):
				if os.path.isdir( path ) == isdir:
					self.store_snapshots.append( [ gnometools.get_snapshot_display_markup( self.snapshots, snapshot ), snapshot ] )
					if snapshot == current_snapshot_id:
						index_combo_diff_with = counter
					counter += 1

		#select first item
		if len( self.store_snapshots ) > 0:
			iter = self.store_snapshots.get_iter_first()
			if not iter is None:
				self.list_snapshots.get_selection().select_iter( iter )
			self.combo_diff_with.set_active( index_combo_diff_with )
	
			self.glade.get_widget( 'btn_diff_with' ).set_sensitive( True )
			self.combo_diff_with.set_sensitive( True )
		else:
			self.glade.get_widget( 'btn_diff_with' ).set_sensitive( False )
			self.combo_diff_with.set_sensitive( False )

		self.list_snapshots.grab_focus()
		self.update_toolbar()

	def on_list_snapshots_row_activated( self, list, path, column ):
		self.open_item()

	def open_item( self ):
		iter = self.list_snapshots.get_selection().get_selected()[1]
		if iter is None:
			return
		path = self.snapshots.get_snapshot_path_to( self.store_snapshots.get_value( iter, 1 ), self.path )
		cmd = "gnome-open \"%s\" &" % path
		os.system( cmd )

	def run( self ):
		snapshot_id = None
		while True:
			ret_val = self.dialog.run()
			
			if gtk.RESPONSE_OK == ret_val: #go to
				iter = self.list_snapshots.get_selection().get_selected()[1]
				if not iter is None:
					snapshot_id = self.store_snapshots.get_value( iter, 1 )
				break
			else:
				#cancel, close ...
				break

		self.dialog.destroy()
		return snapshot_id




#    Back In Time
#    Copyright (C) 2008-2009 Oprea Dan
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os
import os.path
import stat
import sys

if len( os.getenv( 'DISPLAY', '' ) ) == 0:
	os.putenv( 'DISPLAY', ':0.0' )

import pygtk
pygtk.require("2.0")
import gtk
import pynotify
import threading
import gobject
import gtk.glade
import gnome
import gnomevfs
import datetime
import gettext
import time

sys.path = [os.path.join( os.path.dirname( os.path.abspath( os.path.dirname( __file__ ) ) ), 'common' )] + sys.path

import backintime
import config
import logger
import snapshots
import guiapplicationinstance
import tools

import settingsdialog
import snapshotsdialog
import messagebox
import fileicons
import clipboardtools
import gnometools


_=gettext.gettext


class AboutDialog( gtk.AboutDialog ):
	def __init__( self, config, parent ):
		gtk.AboutDialog.__init__( self )
	
		self.connect( 'close', self.on_close )
		self.connect( 'response', self.on_close )

		self.set_transient_for( parent )

		self.set_name( config.APP_NAME )
		self.set_version( config.VERSION )
		self.set_copyright( 'Copyright (C) 2008-2009 Oprea Dan' )
		self.set_website( 'http://backintime.le-web.org' )
		self.set_website_label( 'http://backintime.le-web.org' )
		self.set_license( config.get_license() )
		authors = config.get_authors()
		if not authors is None:
			self.set_authors( authors.split( '\n' ) )

		#extra_translations = _('about-translators').strip()
		#if extra_translations is None:
		#	extra_translations = ''
		#elif extra_translations == 'about-translators':
		#	extra_translations = ''
		#
		#self.set_translator_credits( config.get_translations() + extra_translations )
		self.set_translator_credits( config.get_translations() )

	def on_close( self, *params ):
		self.destroy()


class MainWindow:
	def __init__( self, config, app_instance ):
		self.config = config
		self.app_instance = app_instance
		self.snapshots = snapshots.Snapshots( config )
		self.special_background_color = 'lightblue'
		self.popup_menu = None

		self.folder_path = None
		self.snapshot_id = ''

		self.last_take_snapshot_message = None

		self.glade = gtk.glade.XML( os.path.join( self.config.get_app_path(), 'gnome', 'mainwindow.glade' ), None, 'backintime' )

		signals = { 
				'on_MainWindow_destroy' : gtk.main_quit,
				'on_MainWindow_delete_event' : self.on_close,
				'on_MainWindow_key_release_event': self.on_key_release_event,
				'on_btn_exit_clicked' : self.on_close,
				'on_btn_link_clicked' : self.on_btn_link_clicked,
				'on_btn_help_clicked' : self.on_btn_help_clicked,
				'on_btn_about_clicked' : self.on_btn_about_clicked,
				'on_btn_settings_clicked' : self.on_btn_settings_clicked,
				'on_btn_backup_clicked' : self.on_btn_backup_clicked,
				'on_btn_update_snapshots_clicked' : self.on_btn_update_snapshots_clicked,
				'on_btn_snapshot_name_clicked' : self.on_btn_snapshot_name_clicked,
				'on_btn_remove_snapshot_clicked' : self.on_btn_remove_snapshot_clicked,
				'on_btn_restore_clicked' : self.on_btn_restore_clicked,
				'on_btn_copy_clicked' : self.on_btn_copy_clicked,
				'on_btn_snapshots_clicked' : self.on_btn_snapshots_clicked,
				'on_btn_hidden_files_toggled' : self.on_btn_hidden_files_toggled,
				'on_list_places_cursor_changed' : self.on_list_places_cursor_changed,
				'on_list_time_line_cursor_changed' : self.on_list_time_line_cursor_changed,
				'on_btn_folder_up_clicked' : self.on_btn_fodler_up_clicked,
				'on_list_folder_view_row_activated' : self.on_list_folder_view_row_activated,
				'on_list_folder_view_popup_menu' : self.on_list_folder_view_popup_menu,
				'on_list_folder_view_button_press_event': self.on_list_folder_view_button_press_event,
				'on_list_folder_view_drag_data_get': self.on_list_folder_view_drag_data_get
			}

		self.glade.signal_autoconnect( signals )

		self.window = self.glade.get_widget( 'MainWindow' )
		self.window.set_title( self.config.APP_NAME )

		#icons
		self.icon_names = fileicons.GnomeFileIcons()

		#fix a glade bug
		self.glade.get_widget( 'btn_current_path' ).set_expand( True )
		self.glade.get_widget( 'tb_sep_item' ).set_expand( True )

		#lbl snapshot
		self.lbl_snapshot = self.glade.get_widget( 'lbl_snapshot' )

		#status bar
		self.status_bar = self.glade.get_widget( 'status_bar' )
		self.status_bar.push( 0, _('Done') )

		#show hidden files
		self.show_hidden_files = self.config.get_bool_value( 'gnome.show_hidden_files', False )
		self.btn_hidden_files = self.glade.get_widget( 'btn_hidden_files' )
		self.btn_hidden_files.set_active( self.show_hidden_files )
		#self.btn_hidden_files.set_label( _('Hidden files') ) #!!!

		#setup places view
		self.list_places = self.glade.get_widget( 'list_places' )

		pix_renderer = gtk.CellRendererPixbuf()
		text_renderer = gtk.CellRendererText()
		
		column = gtk.TreeViewColumn( _('Shortcuts') )
		column.pack_start( pix_renderer, False )
		column.pack_end( text_renderer, True )
		column.add_attribute( pix_renderer, 'icon-name', 2 )
		column.add_attribute( text_renderer, 'markup', 0 )
		column.set_cell_data_func( pix_renderer, self.places_pix_renderer_function, None )
		column.set_cell_data_func( text_renderer, self.places_text_renderer_function, None )
		#column.set_alignment( 0.5 )
		self.list_places.append_column( column )

		#name, icon, path
		self.store_places = gtk.ListStore( str, str, str )
		self.list_places.set_model( self.store_places )
		self.list_places.get_selection().set_select_function( self.places_select_function, self.store_places )

		#setup folder view
		self.list_folder_view_widget = self.glade.get_widget( 'list_folder_view_widget' )
		self.list_folder_view_shadow = self.glade.get_widget( 'list_folder_view_shadow' )
		self.list_folder_view = self.glade.get_widget( 'list_folder_view' )

		pix_renderer = gtk.CellRendererPixbuf()
		text_renderer = gtk.CellRendererText()

		column = gtk.TreeViewColumn( _('Name') )
		column.pack_start( pix_renderer, False )
		column.pack_end( text_renderer, True )
		column.add_attribute( pix_renderer, 'icon-name', 2 )
		column.add_attribute( text_renderer, 'markup', 0 )
		column.set_expand( True )
		column.set_sizing( gtk.TREE_VIEW_COLUMN_AUTOSIZE )
		column.set_sort_column_id( 0 )
		self.list_folder_view.append_column( column )

		text_renderer = gtk.CellRendererText()
		column = gtk.TreeViewColumn( _('Size') )
		column.pack_end( text_renderer, True )
		column.add_attribute( text_renderer, 'markup', 4 )
		column.set_sort_column_id( 1 )
		self.list_folder_view.append_column( column )

		text_renderer = gtk.CellRendererText()
		column = gtk.TreeViewColumn( _('Date') )
		column.pack_end( text_renderer, True )
		column.add_attribute( text_renderer, 'markup', 5 )
		column.set_sort_column_id( 2 )
		self.list_folder_view.append_column( column )

		# display name, relative path, icon_name, type (0 - directory, 1 - file), size (str), date, size (int)
		self.store_folder_view = gtk.ListStore( str, str, str, int, str, str, int )
		self.store_folder_view.set_sort_func( 0, self.sort_folder_view_by_column, 0 ) #name
		self.store_folder_view.set_sort_func( 1, self.sort_folder_view_by_column, 6 ) #size
		self.store_folder_view.set_sort_func( 2, self.sort_folder_view_by_column, 5 )	#date
		self.store_folder_view.set_sort_column_id( 0, gtk.SORT_ASCENDING )

		self.list_folder_view.set_model( self.store_folder_view )

		#setup time line view
		self.list_time_line = self.glade.get_widget( 'list_time_line' )

		text_renderer = gtk.CellRendererText()
		column = gtk.TreeViewColumn( _('Snapshots'), text_renderer, markup = 0 )
		column.set_cell_data_func( text_renderer, self.places_text_renderer_function, None )
		self.list_time_line.append_column( column )

		#display name, id
		self.store_time_line = gtk.ListStore( str, str )
		self.list_time_line.set_model( self.store_time_line )
		self.list_time_line.get_selection().set_select_function( self.places_select_function, self.store_time_line )
		self.update_time_line = False

		#calculate specialBackgroundColor
		style = self.list_time_line.get_style()
		bg1 = style.bg[gtk.STATE_NORMAL]
		bg2 = style.bg[gtk.STATE_SELECTED]
		self.special_background_color = gtk.gdk.Color( (2 * bg1.red + bg2.red) / 3, (2 * bg1.green + bg2.green) / 3,(2 * bg1.blue + bg2.blue) / 3 ).to_string()

		#restore size & position
		main_window_x = self.config.get_int_value( 'gnome.main_window.x', -1 )
		main_window_y = self.config.get_int_value( 'gnome.main_window.y', -1 )
		if main_window_x >= 0 and main_window_y >= 0:
			self.window.move( main_window_x, main_window_y )

		main_window_width = self.config.get_int_value( 'gnome.main_window.width', -1 )
		main_window_height = self.config.get_int_value( 'gnome.main_window.height', -1 )
		if main_window_width > 0 and main_window_height > 0:
			self.window.resize( main_window_width, main_window_height )

		main_window_hpaned1 = self.config.get_int_value( 'gnome.main_window.hpaned1', -1 )
		main_window_hpaned2 = self.config.get_int_value( 'gnome.main_window.hpaned2', -1 )
		if main_window_hpaned1 > 0 and main_window_hpaned2 > 0:
			self.glade.get_widget('hpaned1').set_position( main_window_hpaned1 )
			self.glade.get_widget('hpaned2').set_position( main_window_hpaned2 )

		#prepare popup menu ids
		gtk.stock_add( 
				[ ('backintime.open', _('Open'), 0, 0, 'backintime' ),
				  ('backintime.copy', _('Copy'), 0, 0, 'backintime' ),
				  ('backintime.snapshots', _('Snapshots'), 0, 0, 'backintime' ),
				  ('backintime.diff', _('Diff'), 0, 0, 'backintime' ),
				  ('backintime.restore', _('Restore'), 0, 0, 'backintime' ) ] )

		#show main window
		self.window.show()
		gnometools.run_gtk_update_loop()

		self.force_wait_lock_counter = 0
		gobject.timeout_add( 1000, self.raise_application )

		if not self.config.is_configured():
			settingsdialog.SettingsDialog( self.config, self ).run()

			if not self.config.is_configured():
				return 

		if not self.config.can_backup():
			messagebox.show_error( self.window, self.config, _('Can\'t find snapshots folder.\nIf it is on a removable drive please plug it and then press OK') )

		self.update_all( True )

		self.update_backup_info()
		gobject.timeout_add( 1000, self.update_backup_info )

	def sort_folder_view_by_column( self, treemodel, iter1, iter2, column ):
		if 0 == column:
			ascending = 1
			if self.store_folder_view.get_sort_column_id()[1] != gtk.SORT_ASCENDING:
				ascending = -1

			type1 = self.store_folder_view.get_value( iter1, 3 )
			type2 = self.store_folder_view.get_value( iter2, 3 )

			if type1 == 0 and type2 != 0:
				return -1 * ascending

			if type1 != 0 and type2 == 0:
				return 1 * ascending

		data1 = self.store_folder_view.get_value( iter1, column )
		data2 = self.store_folder_view.get_value( iter2, column )

		if type(data1) is str:
			data1 = data1.upper()

		if type(data2) is str:
			data2 = data2.upper()

		#print "sort_folder_view_by_column: " + str( data1 ) + " - " + str( data2 )

		if data1 < data2:
			return -1

		if data1 > data2:
			return 1

		return 0


	def get_default_startup_folder_and_file( self ):
		last_path = self.config.get_str_value( 'gnome.last_path', '' )
		if len(last_path) > 0 and os.path.isdir(last_path):
			return ( last_path, None, False )
		return ( '/', None, False )

	def get_cmd_startup_folder_and_file( self, cmd ):
		if cmd is None:
			cmd = self.app_instance.raise_cmd

		if len( cmd ) < 1:
			return None

		path = None
		show_snapshots = False

		for arg in cmd.split( '\n' ):
			if len( arg ) < 1:
				continue
			if arg == '-s' or arg == '--snapshots':
				show_snapshots = True
				continue
			if arg[0] == '-':
				continue
			if path is None:
				path = arg

		if path is None:
			return None

		if len( path ) < 1:
			return None

		path = os.path.expanduser( path )
		path = os.path.abspath( path )

		if os.path.isdir( path ):
			if len( path ) < 1:
				show_snapshots = False

			if show_snapshots:
				return ( os.path.dirname( path ), path, True )
			else:
				return ( path, '', False )

		if os.path.isfile( path ):
			return ( os.path.dirname( path ), path, show_snapshots )

		return None

	def get_startup_folder_and_file( self, cmd = None ):
		startup_folder = self.get_cmd_startup_folder_and_file( cmd )
		if startup_folder is None:
			return self.get_default_startup_folder_and_file()
		return startup_folder

	def update_all( self, init ):
		#fill lists
		selected_file = None
		show_snapshots = False
		if init:
			self.folder_path, selected_file, show_snapshots = self.get_startup_folder_and_file()
		self.snapshot_id = '/'
		self.snapshots_list = []

		self.fill_places()
		self.fill_time_line( False )
		self.update_folder_view( 1, selected_file, show_snapshots )

	def places_pix_renderer_function( self, column, renderer, model, iter, user_data ):
		if len( model.get_value( iter, 1 ) ) == 0:
			renderer.set_property( 'visible', False )
		else:
			renderer.set_property( 'visible', True )

	def places_text_renderer_function( self, column, renderer, model, iter, user_data ):
		if len( model.get_value( iter, 1 ) ) == 0:
			renderer.set_property( 'cell-background-set', True )
			renderer.set_property( 'cell-background', self.special_background_color )
		else:
			renderer.set_property( 'cell-background-set', False )

	def places_select_function( self, info, store ):
		if len( store.get_value( store.get_iter( info[0] ), 1 ) ) == 0:
			return False
		return True

	def raise_application( self ):
		raise_cmd = self.app_instance.raise_command()
		if raise_cmd is None:
			return True

		print "Raise cmd: " + raise_cmd
		self.window.present_with_time( int(time.time()) )
		self.window.window.focus()
		#self.window.present()

		if len( raise_cmd ) == 0:
			return True

		print "Check if the main window is the only top level visible window"
		for window in gtk.window_list_toplevels():
			if window.get_property( 'visible' ):
				if window != self.window:
					print "Failed"
					return True

		print "OK"

		folder_and_file = self.get_cmd_startup_folder_and_file( raise_cmd )
		if folder_and_file is None:
			return True

		folder_path, file_name, show_snapshots = folder_and_file

		#select now
		self.snapshot_id = '/'
		self.list_time_line.get_selection().select_iter( self.store_time_line.get_iter_first() )

		#select the specified file
		self.folder_path = folder_path
		self.update_folder_view( 1, file_name, show_snapshots )

		return True

	def update_backup_info( self, force_wait_lock = False ):
		if None is self.glade.get_widget( 'btn_backup' ):
			return True

		#print "forceWaitLock: %s" % forceWaitLock

		if force_wait_lock:
			self.force_wait_lock_counter = 10
		
		busy = self.snapshots.is_busy()
		if busy:
			self.force_wait_lock_counter = 0

		if self.force_wait_lock_counter > 0:
			self.force_wait_lock_counter = self.force_wait_lock_counter - 1

		fake_busy = busy or self.force_wait_lock_counter > 0
		self.glade.get_widget( 'btn_backup' ).set_sensitive( not fake_busy )

		if fake_busy:
			take_snapshot_message = None

			if busy:
				take_snapshot_message = self.snapshots.get_take_snapshot_message()

			if take_snapshot_message is None:
				take_snapshot_message = ( 0, '...' )

			if take_snapshot_message != self.last_take_snapshot_message:
				self.last_take_snapshot_message = take_snapshot_message
				self.status_bar.push( 0, _('Working:') + ' ' + self.last_take_snapshot_message[1].replace( '\n', ' ' ) )

			if not self.update_time_line:
				self.update_time_line = True
		elif self.update_time_line:
			self.update_time_line = False
			snapshots_list = self.snapshots_list

			self.fill_time_line()

			#print "New backup: %s" % self.lastBackupList
			#print "Last backup: %s" % lastBackupList

			if snapshots_list != self.snapshots_list:
				self.status_bar.push( 0, _('Done') )
			else:
				self.status_bar.push( 0, _('Done, no backup needed') )

		if not fake_busy:
			self.last_take_snapshot_message = None

		return True

	def fill_places( self ):
		self.store_places.clear()

		#add global places
		self.store_places.append( [ "<b>%s</b>" % _('Global'), '', '' ] )
		self.store_places.append( [ _('Root'), '/', gtk.STOCK_HARDDISK ] )
		self.store_places.append( [ _('Home'), os.path.expanduser( '~' ), gtk.STOCK_HOME ] )

		#add bookmarks
		rawbookmarks = ''
		
		try:
			file = open( os.path.expanduser('~/.gtk-bookmarks') )
			rawbookmarks = file.read()
			file.close()
		except:
			pass

		bookmarks = []
		for rawbookmark in rawbookmarks.split( '\n' ):
			if rawbookmark.startswith( 'file://' ):
				index = rawbookmark.find( ' ' )
				if index > 0:
					bookmarks.append( ( gnomevfs.unescape_string_for_display( rawbookmark[ 7 : index ] ), rawbookmark[ index + 1 : ] ) )
				elif index < 0:
					index = rawbookmark.rfind( '/' )
					if index > 0:
						bookmarks.append( ( gnomevfs.unescape_string_for_display( rawbookmark[ 7 : ] ), gnomevfs.unescape_string_for_display( rawbookmark[ index + 1 : ] ) ) )

		if len( bookmarks ) > 0:
			self.store_places.append( [ "<b>%s</b>" % _('Bookmarks'), '', '' ] )
			for bookmark in bookmarks:
				self.store_places.append( [ bookmark[1], bookmark[0], self.icon_names.get_icon(bookmark[0]) ] )

		#add backup folders
		include_folders = self.config.get_include_folders()
		if len( include_folders ) > 0:
			if len( include_folders ) > 0:
				self.store_places.append( [ "<b>%s</b>" % _('Backup folders'), '', '' ] )
				for folder in include_folders:
					self.store_places.append( [ folder[0], folder[0], gtk.STOCK_SAVE ] )

	def fill_time_line( self, update_folder_view = True ):
		current_selection = '/'
		iter = self.list_time_line.get_selection().get_selected()[1]
		if not iter is None:
			current_selection = self.store_time_line.get_value( iter, 1 )

		self.store_time_line.clear()
		self.store_time_line.append( [ gnometools.get_snapshot_display_markup( self.snapshots, '/' ), '/' ] )

		self.snapshots_list = self.snapshots.get_snapshots_list() 

		groups = []
		now = datetime.date.today()

		#today
		date = now
		groups.append( (_('Today'), self.snapshots.get_snapshot_id( date ), []) )

		#yesterday
		date = now - datetime.timedelta( days = 1 )
		groups.append( (_('Yesterday'), self.snapshots.get_snapshot_id( date ), []) )

		#this week
		date = now - datetime.timedelta( days = now.weekday() )
		groups.append( (_('This week'), self.snapshots.get_snapshot_id( date ), []) )

		#last week
		date = now - datetime.timedelta( days = now.weekday() + 7 )
		groups.append( (_('Last week'), self.snapshots.get_snapshot_id( date ), []) )

		#fill groups
		for snapshot_id in self.snapshots_list:
			found = False

			for group in groups:
				if snapshot_id >= group[1]:
					group[2].append( snapshot_id )
					found = True
					break

			if not found:
				year = int( snapshot_id[ 0 : 4 ] )
				month = int( snapshot_id[ 4 : 6 ] )
				date = datetime.date( year, month, 1 )

				group_name = ''
				if year == now.year:
					group_name = date.strftime( '%B' ).capitalize()
				else:
					group_name = date.strftime( '%B, %Y' ).capitalize()

				groups.append( ( group_name, self.snapshots.get_snapshot_id( date ), [ snapshot_id ]) )

		#fill time_line list
		for group in groups:
			if len( group[2] ) > 0:
				self.store_time_line.append( [ "<b>%s</b>" % group[0], '' ] );
				for snapshot_id in group[2]:
					self.store_time_line.append( [ gnometools.get_snapshot_display_markup( self.snapshots, snapshot_id ), snapshot_id ] )

		#select previous item
		iter = self.store_time_line.get_iter_first()
		while not iter is None:
			if current_selection == self.store_time_line.get_value( iter, 1 ):
				break
			iter = self.store_time_line.iter_next( iter )

		changed = False
		if iter is None:
			changed = True
			iter = self.store_time_line.get_iter_first()

		self.list_time_line.get_selection().select_iter( iter )

		if changed and update_folder_view:
			self.update_folder_view( 2 )

	def on_close( self, *params ):
		main_window_x, main_window_y = self.window.get_position()
		main_window_width, main_window_height = self.window.get_size()
		main_window_hpaned1 = self.glade.get_widget('hpaned1').get_position()
		main_window_hpaned2 = self.glade.get_widget('hpaned2').get_position()

		self.config.set_int_value( 'gnome.main_window.x', main_window_x )
		self.config.set_int_value( 'gnome.main_window.y', main_window_y )
		self.config.set_int_value( 'gnome.main_window.width', main_window_width )
		self.config.set_int_value( 'gnome.main_window.height', main_window_height )
		self.config.set_int_value( 'gnome.main_window.hpaned1', main_window_hpaned1 )
		self.config.set_int_value( 'gnome.main_window.hpaned2', main_window_hpaned2 )
		self.config.set_str_value( 'gnome.last_path', self.folder_path )
		self.config.set_bool_value( 'gnome.show_hidden_files', self.show_hidden_files )

		self.config.save()
		self.window.destroy()
		return True

	def on_list_time_line_cursor_changed( self, list ):
		if list.get_selection().path_is_selected( list.get_cursor()[ 0 ] ):
			self.update_folder_view( 2 )

	def on_list_places_cursor_changed( self, list ):
		if list.get_selection().path_is_selected( list.get_cursor()[ 0 ] ):
			iter = list.get_selection().get_selected()[1]
			folder_path = self.store_places.get_value( iter, 1 )
			if folder_path != self.folder_path:
				self.folder_path = folder_path
				self.update_folder_view( 0 )

	def on_list_folder_view_drag_data_get( self, widget, drag_context, selection_data, info, timestamp, user_param1 = None ):
		iter = self.list_folder_view.get_selection().get_selected()[1]
		if not iter is None:
			path = self.store_folder_view.get_value( iter, 1 )
			path = self.snapshots.get_snapshot_path_to( self.snapshot_id, path )
			path = gnomevfs.escape_path_string( path )
			selection_data.set_uris( [ 'file://' + path ] )

	def on_list_folder_view_button_press_event( self, list, event ):
		if event.button != 3:
			return

		if len( self.store_folder_view ) <= 0:
			return

		path = self.list_folder_view.get_path_at_pos( int( event.x ), int( event.y ) )
		if path is None:
			return
		path = path[0]
	
		self.list_folder_view.get_selection().select_path( path )
		self.show_folder_view_menu_popup( self.list_folder_view, event.button, event.time )

	def on_list_folder_view_popup_menu( self, list ):
		self.show_folder_view_menu_popup( list, 1, gtk.get_current_event_time() )

	def show_folder_view_menu_popup( self, list, button, time ):
		iter = list.get_selection().get_selected()[1]
		if iter is None:
			return

		#print "popup-menu"
		self.popup_menu = gtk.Menu()

		menu_item = gtk.ImageMenuItem( 'backintime.open' )
		menu_item.set_image( gtk.image_new_from_icon_name( self.store_folder_view.get_value( iter, 2 ), gtk.ICON_SIZE_MENU ) )
		menu_item.connect( 'activate', self.on_list_folder_view_open_item )
		self.popup_menu.append( menu_item )

		self.popup_menu.append( gtk.SeparatorMenuItem() )

		menu_item = gtk.ImageMenuItem( 'backintime.copy' )
		menu_item.set_image( gtk.image_new_from_stock( gtk.STOCK_COPY, gtk.ICON_SIZE_MENU ) )
		menu_item.connect( 'activate', self.on_list_folder_view_copy_item )
		self.popup_menu.append( menu_item )

		menu_item = gtk.ImageMenuItem( 'backintime.snapshots' )
		menu_item.set_image( gtk.image_new_from_stock( gtk.STOCK_INDEX, gtk.ICON_SIZE_MENU ) )
		menu_item.connect( 'activate', self.on_list_folder_view_snapshots_item )
		self.popup_menu.append( menu_item )

		if len( self.snapshot_id ) > 1:
			menu_item = gtk.ImageMenuItem( 'backintime.restore' )
			menu_item.set_image( gtk.image_new_from_stock( gtk.STOCK_UNDELETE, gtk.ICON_SIZE_MENU ) )
			menu_item.connect( 'activate', self.on_list_folder_view_restore_item )
			self.popup_menu.append( menu_item )

		self.popup_menu.show_all()
		self.popup_menu.popup( None, None, None, button, time )

	def on_list_folder_view_restore_item( self, widget, data = None ):
		self.on_btn_restore_clicked( self.glade.get_widget( 'btn_restore' ) )

	def on_list_folder_view_copy_item( self, widget, data = None ):
		self.on_btn_copy_clicked( self.glade.get_widget( 'btn_copy' ) )

	def on_list_folder_view_snapshots_item( self, widget, data = None ):
		self.on_btn_snapshots_clicked( self.glade.get_widget( 'btn_snapshots' ) )

	def on_list_folder_view_open_item( self, widget, data = None ):
		iter = self.list_folder_view.get_selection().get_selected()[1]
		if iter is None:
			return

		path = self.store_folder_view.get_value( iter, 1 )
		path = self.snapshots.get_snapshot_path_to( self.snapshot_id, path )
		cmd = "gnome-open \"%s\" &" % path
		print cmd
		os.system( cmd )

	def on_list_folder_view_row_activated( self, list, path, column ):
		iter = list.get_selection().get_selected()[1]
		path = self.store_folder_view.get_value( iter, 1 )

		#directory
		if 0 == self.store_folder_view.get_value( iter, 3 ):
			self.folder_path = path
			self.update_folder_view( 1 )
			return

		#file
		path = self.snapshots.get_snapshot_path_to( self.snapshot_id, path )
		cmd = "gnome-open \"%s\" &" % path
		print cmd
		os.system( cmd )

	def on_btn_fodler_up_clicked( self, button ):
		if len( self.folder_path ) <= 1:
			return

		index = self.folder_path.rfind( '/' )
		if index < 1:
			parent_path = "/"
		else:
			parent_path = self.folder_path[ : index ]

		self.folder_path = parent_path
		self.update_folder_view( 1 )

	def on_btn_restore_clicked( self, button ):
		iter = self.list_folder_view.get_selection().get_selected()[1]
		if not iter is None:
			button.set_sensitive( False )
			gnometools.run_gtk_update_loop()
			self.snapshots.restore( self.snapshot_id, self.store_folder_view.get_value( iter, 1 ) )
			button.set_sensitive( True )
	
	def on_btn_copy_clicked( self, button ):
		iter = self.list_folder_view.get_selection().get_selected()[1]
		if iter is None:
			return

		path = self.store_folder_view.get_value( iter, 1 )
		path = self.snapshots.get_snapshot_path_to( self.snapshot_id, path )

		clipboardtools.clipboard_copy_path( path )

	def on_btn_hidden_files_toggled( self, button ):
		if self.folder_path is None:
			return

		self.show_hidden_files = button.get_active()
		self.update_folder_view( 0 )

	def on_btn_snapshots_clicked( self, button ):
		iter = self.list_folder_view.get_selection().get_selected()[1]
		if iter is None:
			return

		path = self.store_folder_view.get_value( iter, 1 )
		icon_name = self.store_folder_view.get_value( iter, 2 )

		retVal = snapshotsdialog.SnapshotsDialog( self.snapshots, self.window, path, self.snapshots_list, self.snapshot_id, icon_name ).run()
		
		#select the specified file
		if not retVal is None:
			iter = self.store_time_line.get_iter_first()
			while not iter is None:
				snapshot_id = self.store_time_line.get_value( iter, 1 )
				if snapshot_id == retVal:
					break
				iter = self.store_time_line.iter_next( iter )

			if not iter is None:
				self.list_time_line.get_selection().select_iter( iter )
				self.update_folder_view( 2 )

	def on_btn_about_clicked( self, button ):
		AboutDialog( self.config, self.window ).run()

	def on_help( self ):
		gnome.help_display('backintime')

	def on_btn_help_clicked( self, button ):
		self.on_help()

	def on_btn_link_clicked( self, button ):
		os.system( "gnome-open http://backintime.le-web.org &" )

	def on_key_release_event( self, widget, event ):
		if 'F1' == gtk.gdk.keyval_name( event.keyval ) and ( event.state & (gtk.gdk.CONTROL_MASK | gtk.gdk.MOD1_MASK) ) == 0:
			self.on_help()

	def on_btn_settings_clicked( self, button ):
		snapshots_path = self.config.get_snapshots_path()
		include_folders = self.config.get_include_folders()

		settingsdialog.SettingsDialog( self.config, self ).run()

		if snapshots_path != self.config.get_snapshots_path() or include_folders != self.config.get_include_folders():
			self.update_all( False )

	def on_btn_snapshot_name_clicked( self, button ):
		iter = self.list_time_line.get_selection().get_selected()[1]
		if iter is None:
			return

		snapshot_id = self.store_time_line.get_value( iter, 1 )
		if len( snapshot_id ) <= 1:
			return

		old_name = self.snapshots.get_snapshot_name( snapshot_id )
		new_name = messagebox.text_input_dialog( self.window, self.config, _('Snapshot Name'), old_name )
		if new_name is None:
			return

		if old_name == new_name:
			return

		self.snapshots.set_snapshot_name( snapshot_id, new_name )
		self.store_time_line.set_value( iter, 0, gnometools.get_snapshot_display_markup( self.snapshots, snapshot_id ) )

	def on_btn_remove_snapshot_clicked( self, button ):
		iter = self.list_time_line.get_selection().get_selected()[1]
		if iter is None:
			return

		snapshot_id = self.store_time_line.get_value( iter, 1 )
		if len( snapshot_id ) <= 1:
			return

		if gtk.RESPONSE_YES == messagebox.show_question( self.window, self.config, _('Are you sure you want to remove the snapshot:\n<b>%s</b>') % self.snapshots.get_snapshot_display_name( snapshot_id ) ):
			print "Remove Snapshot: %s" % snapshot_id
			self.snapshots.remove_snapshot( snapshot_id )
			self.fill_time_line()

	def on_btn_backup_clicked( self, button ):
		button.set_sensitive( False )
		self.updatetime_line = True
		
		if self.snapshots.is_busy():
			self.update_backup_info()
			return

		backintime.take_snapshot_now_async()
		self.update_backup_info( True )

	def on_btn_update_snapshots_clicked( self, button ):
		self.fill_time_line( False )
		self.update_folder_view( 2 )

	def update_folder_view( self, changed_from, selected_file = None, show_snapshots = False ): #0 - places, 1 - folder view, 2 - time_line
		#update backup time
		if 2 == changed_from:
			iter = self.list_time_line.get_selection().get_selected()[1]
			self.snapshot_id = self.store_time_line.get_value( iter, 1 )
			#self.lblTime.set_markup( "<b>%s</b>" % backupTime )

		tooltip = ''
		text = ''
		if len( self.snapshot_id ) > 1:
			name = self.snapshots.get_snapshot_display_id( self.snapshot_id )
			text = _('Snapshot: %s') % name
			tooltip = _('View the snapshot made at %s') % name
		else:
			tooltip = _('View the current disk content')
			text = _('Now')

		self.lbl_snapshot.set_markup( ' <b>' + text + '</b> ' )
		self.lbl_snapshot.set_tooltip_text( tooltip )

		#update selected places item
		if 1 == changed_from:
			iter = self.store_places.get_iter_first()
			while not iter is None:
				place_path = self.store_places.get_value( iter, 1 )
				if place_path == self.folder_path:
					break
				iter = self.store_places.iter_next( iter )

			if iter is None:
				self.list_places.get_selection().unselect_all()
			else:
				self.list_places.get_selection().select_iter( iter )

		#update folder view
		
		full_path = self.snapshots.get_snapshot_path_to( self.snapshot_id, self.folder_path )
		all_files = []
		files = []
		folder_exists = os.path.isdir( full_path )

		if folder_exists:
			try:
				all_files = os.listdir( full_path )
				all_files.sort()
			except:
				pass

			files = []
			for file in all_files:
				if len( file ) == 0:
					continue

				if not self.show_hidden_files:
					if file[ 0 ] == '.':
						continue
					if file[ -1 ] == '~':
						continue

				path = os.path.join( full_path, file )

				file_size = -1
				file_date = -1

				try:
					file_stat = os.stat( path )
					file_size = file_stat[stat.ST_SIZE]
					file_date = file_stat[stat.ST_MTIME]
				except:
					pass

				#format size
				file_size_int = file_size
				if file_size_int < 0:
					file_size_int = 0

				if file_size < 0:
					file_size = 'unknown'
				elif file_size < 1024:
					file_size = str( file_size ) + ' bytes'
				elif file_size < 1024 * 1024:
					file_size = file_size / 1024
					file_size = str( file_size ) + ' KB'
				elif file_size < 1024 * 1024 * 1024:
					file_size = file_size / ( 1024 * 1024 )
					file_size = str( file_size ) + ' MB'
				else:
					file_size = file_size / ( 1024 * 1024 * 1024 )
					file_size = str( file_size ) + ' GB'

				#format date
				if file_date < 0:
					file_date = 'unknown'
				else:
					file_date = datetime.datetime.fromtimestamp(file_date).isoformat(' ')

				if os.path.isdir( path ):
					files.append( [ file, file_size, file_date, self.icon_names.get_icon(path), file_size_int, 0 ] )
				else:
					files.append( [ file, file_size, file_date, self.icon_names.get_icon(path), file_size_int, 1 ] )

			#try to keep old selected file
			if selected_file is None:
				selected_file = ''
				iter = self.list_folder_view.get_selection().get_selected()[1]
				if not iter is None:
					selected_file = self.store_folder_view.get_value( iter, 1 )

			self.list_folder_view_widget.show()
			self.list_folder_view_shadow.hide()
		else:
			self.list_folder_view_widget.hide()
			self.list_folder_view_shadow.show()

		#populate the list
		self.store_folder_view.clear()

		selected_iter = None
		for item in files:
			rel_path = os.path.join( self.folder_path, item[0] )
			new_iter = self.store_folder_view.append( [ item[0], rel_path, item[3], item[5], item[1], item[2], item[4] ] )
			if selected_file == rel_path:
				selected_iter = new_iter 

		#select old item or the first item
		if len( files ) > 0:
			if selected_iter is None:
				selected_iter = self.store_folder_view.get_iter_first()
			self.list_folder_view.get_selection().select_iter( selected_iter )
			self.list_folder_view.drag_source_set( gtk.gdk.BUTTON1_MASK | gtk.gdk.BUTTON3_MASK, gtk.target_list_add_uri_targets(), gtk.gdk.ACTION_COPY )
		else:
			self.list_folder_view.drag_source_unset()

		#update folderup button state
		self.glade.get_widget( 'btn_folder_up' ).set_sensitive( len( self.folder_path ) > 1 )

		#update restore button state
		self.glade.get_widget( 'btn_restore' ).set_sensitive( len( self.snapshot_id ) > 1 and len( self.store_folder_view ) > 0 )

		#update remove/name snapshot buttons
		self.glade.get_widget( 'btn_snapshot_name' ).set_sensitive( len( self.snapshot_id ) > 1 )
		self.glade.get_widget( 'btn_remove_snapshot' ).set_sensitive( len( self.snapshot_id ) > 1 )

		#update copy button state
		self.glade.get_widget( 'btn_copy' ).set_sensitive( len( self.store_folder_view ) > 0 )

		#update snapshots button state
		self.glade.get_widget( 'btn_snapshots' ).set_sensitive( len( self.store_folder_view ) > 0 )

		#display current folder
		self.glade.get_widget( 'edit_current_path' ).set_text( self.folder_path )

		#show snapshots
		if show_snapshots:
			self.on_btn_snapshots_clicked( None )


def open_url( dialog, link, user_data ):
	os.system( "gnome-open \"%s\" &" % link )


if __name__ == '__main__':
	cfg = backintime.start_app( 'backintime-gnome', ['-s', '--snapshots'] )

	raise_cmd = ''
	if len( sys.argv ) > 1:
		raise_cmd = '\n'.join( sys.argv[ 1 : ] )

	app_instance = guiapplicationinstance.GUIApplicationInstance( cfg.get_app_instance_file(), raise_cmd )

	gnome_props = { gnome.PARAM_APP_DATADIR : '/usr/share' }
	gnome.program_init( 'backintime', cfg.VERSION, properties = gnome_props )

	gtk.about_dialog_set_url_hook( open_url, None )

	logger.openlog()
	main_window = MainWindow( cfg, app_instance )
	if cfg.is_configured():
		gtk.main()
	logger.closelog()

	app_instance.exit_application()




#    Back In Time
#    Copyright (C) 2008-2009 Oprea Dan
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import gtk


def get_snapshot_display_markup( snapshots, snapshot_id ):
	display_name = snapshots.get_snapshot_display_id( snapshot_id )
	name = snapshots.get_snapshot_name( snapshot_id )
	if len( name ) > 0:
		display_name = display_name + ' - <b>' + name + '</b>'
	return display_name


def run_gtk_update_loop():
	gtk.gdk.window_process_all_updates()
		
	while gtk.events_pending():
		gtk.main_iteration( False )




#    Back In Time
#    Copyright (C) 2008-2009 Oprea Dan
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os
import os.path
import time


#class used to handle one application instance mechanism
class ApplicationInstance:

	#specify the file used to save the application instance pid
	def __init__( self, pid_file, auto_exit = True ):
		self.pid_file = pid_file

		if auto_exit:
			if self.check( True ):
				self.start_application()

	#check if the current application is already running, returns True if this is the application instance
	def check( self, auto_exit = False ):
		#check if the pidfile exists
		if not os.path.isfile( self.pid_file ):
			return True

		#read the pid from the file
		pid = 0
		try:
			file = open( self.pid_file, 'rt' )
			data = file.read()
			file.close()
			pid = int( data )
		except:
			pass

		#check if the process with specified by pid exists
		if 0 == pid:
			return True

		try:
			os.kill( pid, 0 )	#this will raise an exception if the pid is not valid
		except:
			return True

		if auto_exit:
			#exit the application
			print "The application is already running !"
			exit(0) #exit raise an exception so don't put it in a try/except block

		return False

	#called when the single instance starts to save it's pid
	def start_application( self ):
		file = open( self.pid_file, 'wt' )
		file.write( str( os.getpid() ) )
		file.close()

	#called when the single instance exit ( remove pid file )
	def exit_application( self ):
		try:
			os.remove( self.pid_file )
		except:
			pass


if __name__ == '__main__':
	#create application instance
	app_instance = ApplicationInstance( '/tmp/myapp.pid' )

	#do something here
	print "Start MyApp"
	time.sleep(5)	#sleep 5 seconds
	print "End MyApp"

	#remove pid file
	app_instance.exit_application()




#    Back In Time
#    Copyright (C) 2008-2009 Oprea Dan
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import pygtk
pygtk.require("2.0")
import gtk
import gnomevfs


def clipboard_set_text( text ):
	clipboard = gtk.clipboard_get()
	clipboard.set_text( text )
	clipboard.store()


def clipboard_copy_path( path ):
	targets = gtk.target_list_add_uri_targets()
	targets = gtk.target_list_add_text_targets( targets)
	targets.append( ( 'x-special/gnome-copied-files', 0, 0 ) )

	clipboard = gtk.clipboard_get()
	clipboard.set_with_data( targets, __clipboard_copy_path_get, __clipboard_copy_path_clear, path )
	clipboard.store()


def __clipboard_copy_path_get( clipboard, selectiondata, info, path ):
	selectiondata.set_text( path )
	path2 = gnomevfs.escape_path_string(path)
	selectiondata.set_uris( [ 'file://' + path2 ] )
	selectiondata.set( 'x-special/gnome-copied-files', 8, 'copy\nfile://' + path2 );

def __clipboard_copy_path_clear( self, path ):
	return




#    Back In Time
#    Copyright (C) 2008-2009 Oprea Dan
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os
import os.path
import stat
import datetime
import gettext
import statvfs
import stat
import bz2
import pwd
import grp

import config
import configfile
import logger
import applicationinstance
import tools
import pluginmanager


_=gettext.gettext


class Snapshots:
	def __init__( self, cfg = None ):
		self.config = cfg
		if self.config is None:
			self.config = config.Config()

		self.plugin_manager = pluginmanager.PluginManager()

	def get_snapshot_id( self, date ):
		if type( date ) is datetime.datetime:
			return date.strftime( '%Y%m%d-%H%M%S' )

		if type( date ) is datetime.date:
			return date.strftime( '%Y%m%d-000000' )

		if type( date ) is str:
			return date
		
		return ""

	def get_snapshot_path( self, date ):
		return os.path.join( self.config.get_snapshots_full_path(), self.get_snapshot_id( date ) )

	def get_snapshot_info_path( self, date ):
		return os.path.join( self.get_snapshot_path( date ), 'info' )

	def get_snapshot_fileinfo_path( self, date ):
		return os.path.join( self.get_snapshot_path( date ), 'fileinfo.bz2' )

	def _get_snapshot_data_path( self, snapshot_id ):
		if len( snapshot_id ) <= 1:
			return '/';
		return os.path.join( self.get_snapshot_path( snapshot_id ), 'backup' )
	
	def get_snapshot_path_to( self, snapshot_id, toPath = '/' ):
		return os.path.join( self._get_snapshot_data_path( snapshot_id ), toPath[ 1 : ] )

	def get_snapshot_display_id( self, snapshot_id ):
		if len( snapshot_id ) <= 1:
			return _('Now')
		return "%s-%s-%s %s:%s:%s" % ( snapshot_id[ 0 : 4 ], snapshot_id[ 4 : 6 ], snapshot_id[ 6 : 8 ], snapshot_id[ 9 : 11 ], snapshot_id[ 11 : 13 ], snapshot_id[ 13 : 15 ]  )
	
	def get_snapshot_display_name( self, snapshot_id ):
		display_name = self.get_snapshot_display_id( snapshot_id )
		name = self.get_snapshot_name( snapshot_id )
		if len( name ) > 0:
			display_name = display_name + ' - ' + name
		return display_name

	def get_snapshot_name( self, snapshot_id ):
		if len( snapshot_id ) <= 1: #not a snapshot
			return ''

		path = self.get_snapshot_path( snapshot_id )
		if not os.path.isdir( path ):
			return ''
		
		name = ''
		try:
			file = open( os.path.join( path, 'name' ), 'rt' )
			name = file.read()
			file.close()
		except:
			pass

		return name

	def set_snapshot_name( self, snapshot_id, name ):
		if len( snapshot_id ) <= 1: #not a snapshot
			return

		path = self.get_snapshot_path( snapshot_id )
		if not os.path.isdir( path ):
			return

		name_path = os.path.join( path, 'name' )

		info_file = configfile.ConfigFile()
		info_file.load( self.get_snapshot_info_path( snapshot_id ) )
		if info_file.get_int_value( 'snapshot_version' ) == 0:
			os.system( "chmod a+w \"%s\"" % path )

		try:
			file = open( name_path, 'wt' )
			file.write( name )
			file.close()
		except:
			pass

	def get_take_snapshot_message( self ):
		try:
			file = open( self.config.get_take_snapshot_message_file(), 'rt' )
			items = file.read().split( '\n' )
			file.close()
		except:
			return None 

		if len( items ) < 2:
			return None

		id = 0
		try:
			id = int( items[0] )
		except:
			pass

		del items[0]
		message = '\n'.join( items )

		return( id, message )

	def set_take_snapshot_message( self, type_id, message ):
		data = str(type_id) + '\n' + message

		try:
			file = open( self.config.get_take_snapshot_message_file(), 'wt' )
			items = file.write( data )
			file.close()
			#logger.info( "Take snapshot message: %s" % data )
		except:
			pass

	def clear_take_snapshot_message( self ):
		os.system( "rm \"%s\"" % self.config.get_take_snapshot_message_file() )

	def is_busy( self ):
		instance = applicationinstance.ApplicationInstance( self.config.get_take_snapshot_instance_file(), False )
		return not instance.check()

	def load_fileinfo_dict( self, snapshot_id, version = None ):
		if version is None:
			info_file = configfile.ConfigFile()
			info_file.load( self.get_snapshot_info_path( snapshot_id ) )
			version = info_file.get_int_value( 'snapshot_version' )
			info_file = None

		dict = {}

		if 0 == version:
			return file_info_dict

		fileinfo_path = self.get_snapshot_fileinfo_path( snapshot_id )
		if not os.path.exists( fileinfo_path ):
			return dict

		#try:
		fileinfo = bz2.BZ2File( fileinfo_path, 'r' )
		while True:
			line = fileinfo.readline()
			if len( line ) <= 0:
				break

			line = line[ : -1 ]
			if len( line ) <= 0:
				continue
		
			index = line.find( '/' )
			if index < 0:
				continue

			file = line[ index: ]
			if len( file ) <= 0:
				continue

			info = line[ : index ].strip()
			info = info.split( ' ' )

			if len( info ) == 3:
				dict[ file ] = [ int( info[0] ), info[1], info[2] ] #perms, user, group

		fileinfo.close()
		#except:
		#	pass

		return dict

	def _restore_path_info( self, path, dict ):
		if path not in dict:
			return

		info = dict[path]

		#restore perms
		try:
			os.chmod( path, info[0] )
		except:
			pass

		#restore uid/gid
		uid = -1
		gid = -1

		if info[1] != '-':
			try:
				uid = pwd.getpwnam( info[1] ).pw_uid
			except:
				pass

		if info[2] != '-':
			try:
				gid = grp.getgrnam( info[2] ).gr_gid
			except:
				pass

		if uid != -1 or gid != -1:
			try:
				os.chown( path, uid, gid )
			except:
				pass

	def restore( self, snapshot_id, path ):
		logger.info( "Restore: %s" % path )

		info_file = configfile.ConfigFile()
		info_file.load( self.get_snapshot_info_path( snapshot_id ) )

		backup_suffix = '.backup.' + datetime.date.today().strftime( '%Y%m%d' )
		#cmd = "rsync -avR --copy-unsafe-links --whole-file --backup --suffix=%s --chmod=+w %s/.%s %s" % ( backup_suffix, self.get_snapshot_path_to( snapshot_id ), path, '/' )
		cmd = "rsync -avRAXE --whole-file --backup --suffix=%s " % backup_suffix
		cmd = cmd + '--chmod=+w '
		cmd = cmd + "\"%s.%s\" %s" % ( self.get_snapshot_path_to( snapshot_id ), path, '/' )
		self._execute( cmd )

		#restore permissions
		file_info_dict = self.load_fileinfo_dict( snapshot_id, info_file.get_int_value( 'snapshot_version' ) )
		if len( file_info_dict ) > 0:
			#explore items
			snapshot_path_to = self.get_snapshot_path_to( snapshot_id, path ).rstrip( '/' )
			root_snapshot_path_to = self.get_snapshot_path_to( snapshot_id ).rstrip( '/' )
			all_dirs = [] #restore dir permissions after all files are done

			path_items = path.strip( '/' ).split( '/' )
			curr_path = '/'
			for path_item in path_items:
				curr_path = os.path.join( curr_path, path_item )
				all_dirs.append( curr_path )

			if not os.path.isfile( snapshot_path_to ):
				for explore_path, dirs, files in os.walk( snapshot_path_to ):
					for item in dirs:
						item_path = os.path.join( explore_path, item )[ len( root_snapshot_path_to ) : ]
						all_dirs.append( item_path )

					for item in files:
						item_path = os.path.join( explore_path, item )[ len( root_snapshot_path_to ) : ]
						self._restore_path_info( item_path, file_info_dict )

			all_dirs.reverse()
			for item_path in all_dirs:
				self._restore_path_info( item_path, file_info_dict )

	def get_snapshots_list( self, sort_reverse = True ):
		biglist = []
		snapshots_path = self.config.get_snapshots_full_path()

		try:
			biglist = os.listdir( snapshots_path )
		except:
			pass

		list = []

		for item in biglist:
			if len( item ) != 15:
				continue
			if os.path.isdir( os.path.join( snapshots_path, item ) ):
				list.append( item )

		list.sort( reverse = sort_reverse )
		return list

	def remove_snapshot( self, snapshot_id ):
		if len( snapshot_id ) <= 1:
			return

		path = self.get_snapshot_path( snapshot_id )
		cmd = "chmod -R a+rwx \"%s\"" %  path
		self._execute( cmd )
		cmd = "rm -rfv \"%s\"" % path
		self._execute( cmd )

	def _get_last_snapshot_info( self ):
		lines = ''
		dict = {}

		try:
			if os.path.exists( self.config.get_last_snapshot_info_file() ):
				file = open( self.config.get_last_snapshot_info_file(), 'rt' )
				lines = file.read()
				file.close()
		except:
			pass

		lines = lines.split( '\n' )
		for line in lines:
			line = line.strip()
			if len( line ) <= 0:
				continue
			fields = line.split( ':' )
			if len( fields ) < 6:
				continue

			dict[ fields[0] ] = datetime.datetime( int(fields[1]), int(fields[2]), int(fields[3]), int(fields[4]), int(fields[5]) )

		return dict

	def _set_last_snapshot_info( self, dict ):
		lines = []

		for key, value in dict.items():
			lines.append( "%s:%s:%s:%s:%s:%s" % ( key, value.year, value.month, value.day, value.hour, value.minute ) )

		try:
			file = open( self.config.get_last_snapshot_info_file(), 'wt' )
			file.write( '\n'.join( lines ) )
			file.close()
		except:
			pass

	#def call_user_callback( self, args ):
	#	cmd = self.config.get_take_snapshot_user_callback()
	#	if os.path.isfile( cmd ):
	#		self._execute( 'sh ' + cmd + ' ' + args )

	def take_snapshot( self, force = False ):
		ret_val = False
		sleep = True

		self.plugin_manager.load_plugins( self )

		if not self.config.is_configured():
			logger.warning( 'Not configured' )
			self.plugin_manager.on_error( 1 ) #not configured
		else:
			instance = applicationinstance.ApplicationInstance( self.config.get_take_snapshot_instance_file(), False )
			if not instance.check():
				logger.warning( 'A backup is already running' )
				self.plugin_manager.on_error( 2 ) #a backup is already running
			else:
				instance.start_application()
				logger.info( 'Lock' )

				if not self.config.get_per_directory_schedule():
					force = True

				now = datetime.datetime.today()
				if not force:
					now = now.replace( second = 0 )

				include_folders, ignore_folders, dict = self._get_backup_folders( now, force )

				if len( include_folders ) <= 0:
					logger.info( 'Nothing to do' )
				else:
					self.plugin_manager.on_process_begins() #take snapshot process begin

					self.set_take_snapshot_message( 0, '...' )

					if not self.config.can_backup():
						if self.plugin_manager.has_gui_plugins() and self.config.is_notify_enabled():
							for counter in xrange( 30, 0, -1 ):
								self.set_take_snapshot_message( 1, 
										_('Can\'t find snapshots folder.\nIf it is on a removable drive please plug it.' ) +
										'\n' +
										gettext.ngettext( 'Waiting %s second.', 'Waiting %s seconds.', counter ) % counter )
								os.system( 'sleep 1' )
								if self.config.can_backup():
									break

					if not self.config.can_backup():
						logger.warning( 'Can\'t find snapshots folder !' )
						self.plugin_manager.on_error( 3 ) #Can't find snapshots directory (is it on a removable drive ?)
					else:
						snapshot_id = self.get_snapshot_id( now )
						snapshot_path = self.get_snapshot_path( snapshot_id )

						if os.path.exists( snapshot_path ):
							logger.warning( "Snapshot path \"%s\" already exists" % snapshot_path )
							self.plugin_manager.on_error( 4, snapshot_id ) #This snapshots already exists
						else:
							ret_val = self._take_snapshot( snapshot_id, now, include_folders, ignore_folders, dict, force )

						if not ret_val:
							os.system( "rm -rf \"%s\"" % snapshot_path )
							logger.warning( "No new snapshot (not needed or error)" )
						
						self._free_space( now )

						self.set_take_snapshot_message( 0, _('Finalizing') )

					os.system( 'sleep 2' )
					sleep = False

					if ret_val:
						self.plugin_manager.on_new_snapshot( snapshot_id, snapshot_path ) #new snapshot

					self.plugin_manager.on_process_ends() #take snapshot process end

				if sleep:
					os.system( 'sleep 2' )
					sleep = False

				self.clear_take_snapshot_message()
				instance.exit_application()
				logger.info( 'Unlock' )

		if sleep:
			os.system( 'sleep 2' ) #max 1 backup / second

		return ret_val

	def _exec_rsync_callback( self, line, user_data ):
		self.set_take_snapshot_message( 0, _('Take snapshot') + " (rsync: %s)" % line )

	def _exec_rsync_compare_callback( self, line, user_data ):
		self.set_take_snapshot_message( 0, _('Compare with snapshot %s') % user_data + " (rsync: %s)"% line )

	def _append_item_to_list( self, item, list ):
		for list_item in list:
			if item == list_item:
				return
		list.append( item )

	def _is_auto_backup_needed( self, now, last, mode ):
		#print "now: %s, last: %s, mode: %s" % ( now, last, mode )

		if self.config.NONE == mode:
			return False

		if now <= last:
			return False

		if now.year > last.year: #year changed
			return True

		if self.config.MONTH == mode: #month changed
			return now.month > last.month
		
		if self.config.WEEK == mode: #weekly
			if now.date() <= last.date():
				return False
			return now.isoweekday() == 7 #Sunday

		if now.date() > last.date(): #day changed
			return True

		if self.config.DAY == mode:
			return False

		if now.hour > last.hour: #hour changed
			return True
		
		if self.config.HOUR == mode:
			return False

		if self.config._10_MIN == mode: #every 10 minutes
			return ( int( now.minute / 10 ) ) > ( int( last.minute / 10 ) )

		if self.config._5_MIN == mode: #every 5 minutes
			return ( int( now.minute / 5 ) ) > ( int( last.minute / 5 ) )

		return False

	def _get_backup_folders( self, now, force ):
		include_folders = []
		ignore_folders = []
		dict = self._get_last_snapshot_info()
		dict2 = {}

		all_include_folders = self.config.get_include_folders()
		
		for item in all_include_folders:
			path = item[0]
			path = os.path.expanduser( path )
			path = os.path.abspath( path )

			if path in dict:
				dict2[ path ] = dict[ path ]

			if not os.path.isdir( path ):
				continue

			if not force and path in dict:
				if not self._is_auto_backup_needed( now, dict[path], item[1] ):
					ignore_folders.append( path )
					continue

			include_folders.append( path )

		logger.info( "Include folders: %s" % include_folders )
		logger.info( "Ignore folders: %s" % ignore_folders )
		logger.info( "Last snapshots: %s" % dict2 )

		return ( include_folders, ignore_folders, dict2 )

	def _create_directory( self, folder ):
		tools.make_dirs( folder )

		if not os.path.exists( folder ):
			logger.error( "Can't create folder: %s" % folder )
			self.set_take_snapshot_message( 1, _('Can\'t create folder: %s') % folder )
			os.system( 'sleep 2' ) #max 1 backup / second
			return False

		return True
	
	def _save_path_info_line( self, fileinfo, path, info ):
		fileinfo.write( "%s %s %s %s\n" % ( info[0], info[1], info[2], path ) )

	def _save_path_info( self, fileinfo, path ):
		try:
			info = os.stat( path )
			user = '-'
			group = '-'

			try:
				user = pwd.getpwuid( info.st_uid ).pw_name
			except:
				pass

			try:
				group = grp.getgrgid( info.st_gid ).gr_name
			except:
				pass

			self._save_path_info_line( fileinfo, path, [ info.st_mode, user, group ] )
		except:
			pass

	def _take_snapshot( self, snapshot_id, now, include_folders, ignore_folders, dict, force ):
		self.set_take_snapshot_message( 0, _('...') )

		new_snapshot_id = 'new_snapshot'
		new_snapshot_path = self.get_snapshot_path( new_snapshot_id )

		if os.path.exists( new_snapshot_path ):
			#self._execute( "find \"%s\" -type d -exec chmod +w {} \;" % new_snapshot_path )
			self._execute( "chmod -R a+rwx \"%s\"" %  new_snapshot_path )
			self._execute( "rm -rf \"%s\"" % new_snapshot_path )
		
			if os.path.exists( new_snapshot_path ):
				logger.error( "Can't remove folder: %s" % new_snapshot_path )
				self.set_take_snapshot_message( 1, _('Can\'t remove folder: %s') % new_snapshot_path )
				os.system( 'sleep 2' ) #max 1 backup / second
				return False

		new_snapshot_path_to = self.get_snapshot_path_to( new_snapshot_id )

		#create exclude patterns string
		items = []
		for exclude in self.config.get_exclude_patterns():
			self._append_item_to_list( "--exclude=\"%s\"" % exclude, items )
		for folder in ignore_folders:
			self._append_item_to_list( "--exclude=\"%s\"" % folder, items )
		rsync_exclude = ' '.join( items )

		#create include patterns list
		items = []
		items2 = []
		for include_folder in include_folders:
			self._append_item_to_list( "--include=\"%s/**\"" % include_folder, items2 )
			while True:
				self._append_item_to_list( "--include=\"%s/\"" % include_folder, items )
				include_folder = os.path.split( include_folder )[0]
				if len( include_folder ) <= 1:
					break
		rsync_include = ' '.join( items )
		rsync_include2 = ' '.join( items2 )

		#rsync prefix & suffix
		rsync_prefix = 'rsync -aEAX '
		rsync_exclude_backup_directory = " --exclude=\"%s\" --exclude=\"%s\" " % ( self.config.get_snapshots_path(), self.config._LOCAL_DATA_FOLDER )
		rsync_suffix = ' --chmod=Fa-w,D+w --whole-file --delete ' + rsync_exclude_backup_directory  + rsync_include + ' ' + rsync_exclude + ' ' + rsync_include2 + ' --exclude=\"*\" / '

		#update dict
		if not force:
			for folder in include_folders:
				dict[ folder ] = now

			self._set_last_snapshot_info( dict )

		#check previous backup
		snapshots = self.get_snapshots_list()
		prev_snapshot_id = ''

		if len( snapshots ) > 0:
			prev_snapshot_id = snapshots[0]
			prev_snapshot_name = self.get_snapshot_display_id( prev_snapshot_id )
			self.set_take_snapshot_message( 0, _('Compare with snapshot %s') % prev_snapshot_name )
			logger.info( "Compare with old snapshot: %s" % prev_snapshot_id )
			
			prev_snapshot_folder = self.get_snapshot_path_to( prev_snapshot_id )
			cmd = rsync_prefix + ' -i --dry-run ' + rsync_suffix + '"' + prev_snapshot_folder + '"'
			try_cmd = self._execute_output( cmd, self._exec_rsync_compare_callback, prev_snapshot_name )
			changed = False

			for line in try_cmd.split( '\n' ):
				if len( line ) < 1:
					continue

				if line[0] != '.':
					changed = True
					break

			if not changed:
				logger.info( "Nothing changed, no back needed" )
				return False

			if not self._create_directory( new_snapshot_path_to ):
				return False
			
			self.set_take_snapshot_message( 0, _('Create hard-links') )
			logger.info( "Create hard-links" )
			
			if force or len( ignore_folders ) == 0:
				cmd = "cp -al \"%s\"* \"%s\"" % ( self.get_snapshot_path_to( prev_snapshot_id ), new_snapshot_path_to )
				self._execute( cmd )
			else:
				for folder in include_folders:
					prev_path = self.get_snapshot_path_to( prev_snapshot_id, folder )
					new_path = self.get_snapshot_path_to( new_snapshot_id, folder )
					tools.make_dirs( new_path )
					cmd = "cp -alb \"%s\"* \"%s\"" % ( prev_path, new_path )
					self._execute( cmd )
		else:
			if not self._create_directory( new_snapshot_path_to ):
				return False

		#sync changed folders
		logger.info( "Call rsync to take the snapshot" )
		cmd = rsync_prefix + ' -v --delete-excluded ' + rsync_suffix + '"' + new_snapshot_path_to + '"'
		self.set_take_snapshot_message( 0, _('Take snapshot') )
		self._execute( cmd, self._exec_rsync_callback )

		#save permissions for sync folders
		logger.info( 'Save permissions' )
		self.set_take_snapshot_message( 0, _('Save permission ...') )

		fileinfo = bz2.BZ2File( self.get_snapshot_fileinfo_path( new_snapshot_id ), 'w' )
		path_to_explore = self.get_snapshot_path_to( new_snapshot_id ).rstrip( '/' )
		fileinfo_dict = {}

		for path, dirs, files in os.walk( path_to_explore ):
			dirs.extend( files )
			for item in dirs:
				item_path = os.path.join( path, item )[ len( path_to_explore ) : ]
				fileinfo_dict[item_path] = 1
				self._save_path_info( fileinfo, item_path )

		#copy ignored folders
		if not force and len( prev_snapshot_id ) > 0 and len( ignore_folders ) > 0:
			prev_fileinfo_dict = self.load_fileinfo_dict( prev_snapshot_id )

			for folder in ignore_folders:
				prev_path = self.get_snapshot_path_to( prev_snapshot_id, folder )
				new_path = self.get_snapshot_path_to( new_snapshot_id, folder )
				tools.make_dirs( new_path )
				cmd = "cp -alb \"%s/\"* \"%s\"" % ( prev_path, new_path )
				self._execute( cmd )
		
				if len( prev_fileinfo_dict ) > 0:
					#save permissions for all items to folder
					item_path = '/'
					prev_path_items = folder.strip( '/' ).split( '/' )
					for item in items:
						item_path = os.path.join( item_path, item )
						if item_path not in fileinfo_dict and item_path in prev_fileinfo_dict:
							self._save_path_info_line( fileinfo, item_path, prev_fileinfo_dict[item_path] )

					#save permission for all items in folder
					for path, dirs, files in os.walk( new_path ):
						dirs.extend( files )
						for item in dirs:
							item_path = os.path.join( path, item )[ len( path_to_explore ) : ]
							if item_path not in fileinfo_dict and item_path in prev_fileinfo_dict:
								self._save_path_info_line( fileinfo, item_path, prev_fileinfo_dict[item_path] )

		fileinfo.close()

		#create info file
		info_file = configfile.ConfigFile()
		info_file.set_int_value( 'snapshot_version', 1 )
		info_file.save( self.get_snapshot_info_path( new_snapshot_id ) )
		info_file = None

		#rename snapshot
		snapshot_path = self.get_snapshot_path( snapshot_id )
		os.system( "mv \"%s\" \"%s\"" % ( new_snapshot_path, snapshot_path ) )
		if not os.path.exists( snapshot_path ):
			logger.error( "Can't rename %s to %s" % ( new_snapshot_path, snapshot_path ) )
			self.set_take_snapshot_message( 1, _('Can\'t rename %s to %s') % ( new_snapshot_path, snapshot_path ) )
			os.system( 'sleep 2' ) #max 1 backup / second
			return False

		#make new snapshot read-only
		#self._execute( "chmod -R a-w \"%s\"" % snapshot_path )

		return True

	def _smart_remove_keep_all_( self, snapshots, keep_snapshots, min_date ):
		min_id = self.get_snapshot_id( min_date )

		logger.info( "[smart remove] keep all >= %s" % min_id )

		for snapshot_id in snapshots:
			if snapshot_id >= min_id:
				if snapshot_id not in keep_snapshots:
					keep_snapshots.append( snapshot_id )

		return keep_snapshots

	def _smart_remove_keep_first_( self, snapshots, keep_snapshots, min_date, max_date ):
		max_id = self.get_snapshot_id( max_date + datetime.timedelta( days = 1 ) )
		min_id = self.get_snapshot_id( min_date )

		logger.info( "[smart remove] keep first >= %s and < %s" % ( min_id, max_id ) )

		for snapshot_id in snapshots:
			if snapshot_id >= min_id and snapshot_id < max_id:
				if snapshot_id not in keep_snapshots:
					keep_snapshots.append( snapshot_id )
				break

		return keep_snapshots

	def smart_remove( self, now_full = None ):
		snapshots = self.get_snapshots_list()
		if len( snapshots ) <= 1:
			logger.info( "[smart remove] There is only one snapshots, so keep it" )
			return

		if now_full is None:
			now_full = datetime.datetime.today()

		now = datetime.datetime( now_full.year, now_full.month, now_full.day )

		#keep the last snapshot
		keep_snapshots = [ snapshots[0] ]

		#keep all from today and yesterday
		keep_snapshots = self._smart_remove_keep_all_( snapshots, keep_snapshots, now - datetime.timedelta( days = 1 ) )

		#last week	
		max_date = now - datetime.timedelta( days = now.weekday() + 1 )
		min_date = max_date - datetime.timedelta( days = 6 )
		keep_snapshots = self._smart_remove_keep_first_( snapshots, keep_snapshots, min_date, max_date )

		#2 weeks ago
		max_date = max_date - datetime.timedelta( days = 7 )
		min_date = min_date - datetime.timedelta( days = 7 )
		keep_snapshots = self._smart_remove_keep_first_( snapshots, keep_snapshots, min_date, max_date )

		#one per month for all months of this year
		if now.date > 1:
			for month in xrange( 1, now.month ):
				#print "month: %s" % month
				min_date = datetime.date( now.year, month, 1 )
				max_date = datetime.date( now.year, month + 1, 1 ) - datetime.timedelta( days = 1 )
				keep_snapshots = self._smart_remove_keep_first_( snapshots, keep_snapshots, min_date, max_date )

		#one per year for all previous years
		min_year = int( snapshots[ -1 ][ :4 ] )
		for year in xrange( min_year, now.year ):
			#print "year: %s" % year
			min_date = datetime.date( year, 1, 1 )
			max_date = datetime.date( year + 1, 1, 1 ) - datetime.timedelta( days = 1 )
			keep_snapshots = self._smart_remove_keep_first_( snapshots, keep_snapshots, min_date, max_date )

		logger.info( "[smart remove] keep snapshots: %s" % keep_snapshots )

		for snapshot_id in snapshots:
			if snapshot_id in keep_snapshots:
				continue

			if self.config.get_dont_remove_named_snapshots():
				if len( self.get_snapshot_name( snapshot_id ) ) > 0:
					logger.info( "[smart remove] keep snapshot: %s, it has a name" % snapshot_id )
					continue

			logger.info( "[smart remove] remove snapshot: %s" % snapshot_id )
			self.remove_snapshot( snapshot_id )

	def _free_space( self, now ):
		#remove old backups
		if self.config.is_remove_old_snapshots_enabled():
			self.set_take_snapshot_message( 0, _('Remove old snapshots') )
			snapshots = self.get_snapshots_list( False )

			old_backup_id = self.get_snapshot_id( self.config.get_remove_old_snapshots_date() )
			logger.info( "Remove backups older than: %s" % old_backup_id )

			while True:
				if len( snapshots ) <= 1:
					break

				if snapshots[0] >= old_backup_id:
					break

				if self.config.get_dont_remove_named_snapshots():
					if len( self.get_snapshot_name( snapshots[0] ) ) > 0:
						del snapshots[0]
						continue

				self.remove_snapshot( snapshots[0] )
				del snapshots[0]

		#smart remove
		if self.config.get_smart_remove():
			self.set_take_snapshot_message( 0, _('Smart remove') )
			self.smart_remove( now )

		#try to keep min free space
		if self.config.is_min_free_space_enabled():
			self.set_take_snapshot_message( 0, _('Try to keep min free space') )

			min_free_space = self.config.get_min_free_space_in_mb()

			logger.info( "Keep min free disk space: %s Mb" % min_free_space )

			snapshots = self.get_snapshots_list( False )

			while True:
				if len( snapshots ) <= 1:
					break

				info = os.statvfs( self.config.get_snapshots_path() )
				free_space = info[ statvfs.F_FRSIZE ] * info[ statvfs.F_BAVAIL ] / ( 1024 * 1024 )

				if free_space >= min_free_space:
					break

				if self.config.get_dont_remove_named_snapshots():
					if len( self.get_snapshot_name( snapshots[0] ) ) > 0:
						del snapshots[0]
						continue

				logger.info( "free disk space: %s Mb" % free_space )
				self.remove_snapshot( snapshots[0] )
				del snapshots[0]

	def _execute( self, cmd, callback = None, user_data = None ):
		ret_val = 0

		if callback is None:
			ret_val = os.system( cmd )
		else:
			pipe = os.popen( cmd, 'r' )

			while True:
				line = pipe.readline()
				if len( line ) == 0:
					break
				callback( line.strip(), user_data )

			ret_val = pipe.close()
			if ret_val is None:
				ret_val = 0

		if ret_val != 0:
			logger.warning( "Command \"%s\" returns %s" % ( cmd, ret_val ) )
		else:
			logger.info( "Command \"%s\" returns %s" % ( cmd, ret_val ) )

		return ret_val

	def _execute_output( self, cmd, callback = None, user_data = None ):
		output = ''

		pipe = os.popen( cmd, 'r' )
		
		while True:
			line = pipe.readline()
			if len( line ) == 0:
				break
			output = output + line
			if not callback is None:
				callback( line.strip(), user_data )

		ret_val = pipe.close()
		if ret_val is None:
			ret_val = 0

		if ret_val != 0:
			logger.warning( "Command \"%s\" returns %s" % ( cmd, ret_val ) )
		else:
			logger.info( "Command \"%s\" returns %s" % ( cmd, ret_val ) )

		return output


if __name__ == "__main__":
	config = config.Config()
	snapshots = Snapshots( config )
	snapshots.take_snapshot()




#    Back In Time
#    Copyright (C) 2008-2009 Oprea Dan
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os
import os.path
import time


#class used to handle one application instance mechanism
class GUIApplicationInstance:

	#specify the base for control files
	def __init__( self, base_control_file, raise_cmd = '' ):
		self.pid_file = base_control_file + '.pid'
		self.raise_file = base_control_file + '.raise'
		self.raise_cmd = raise_cmd

		#remove raise_file is already exists
		try:
			os.remove( self.raise_file )
		except:
			pass

		self.check( raise_cmd )
		self.start_application()

	#check if the current application is already running
	def check( self, raise_cmd ):
		#check if the pidfile exists
		if not os.path.isfile( self.pid_file ):
			return

		#read the pid from the file
		pid = 0
		try:
			file = open( self.pid_file, 'rt' )
			data = file.read()
			file.close()
			pid = int( data )
		except:
			pass

		#check if the process with specified by pid exists
		if 0 == pid:
			return

		try:
			os.kill( pid, 0 )	#this will raise an exception if the pid is not valid
		except:
			return

		#exit the application
		print "The application is already running ! (pid: %s)" % pid

		#notify raise
		try:
			file = open( self.raise_file, 'wt' )
			file.write( raise_cmd )
			file.close()
		except:
			pass

		exit(0) #exit raise an exception so don't put it in a try/except block

	#called when the single instance starts to save it's pid
	def start_application( self ):
		file = open( self.pid_file, 'wt' )
		file.write( str( os.getpid() ) )
		file.close()

	#called when the single instance exit ( remove pid file )
	def exit_application( self ):
		try:
			os.remove( self.pid_file )
		except:
			pass

	#check if the application must to be raised
	#return None if no raise needed, or a string command to raise
	def raise_command( self ):
		ret_val = None

		try:
			if os.path.isfile( self.raise_file ):
				file = open( self.raise_file, 'rt' )
				ret_val = file.read()
				file.close()
				os.remove( self.raise_file )
		except:
			pass

		return ret_val




#    Back In Time
#    Copyright (C) 2008-2009 Oprea Dan
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os
import os.path
import sys
import pygtk
pygtk.require("2.0")
import gtk
import gobject
import gtk.glade
import datetime
import gettext

import config
import messagebox


_=gettext.gettext


class SettingsDialog:
	def __init__( self, config, parent ):
		self.config = config
		self.parent = parent

		self.glade = gtk.glade.XML( os.path.join( self.config.get_app_path(), 'gnome', 'settingsdialog.glade' ), None, 'backintime' )
		self.dialog = self.glade.get_widget( 'SettingsDialog' )
		self.dialog.set_transient_for( parent.window )

		signals = { 
				'on_btn_add_include_clicked' : self.on_add_include,
				'on_btn_remove_include_clicked' : self.on_remove_include,
				'on_btn_add_exclude_clicked' : self.on_add_exclude,
				'on_btn_add_exclude_file_clicked' : self.on_add_exclude_file,
				'on_btn_add_exclude_folder_clicked' : self.on_add_exclude_folder,
				'on_btn_remove_exclude_clicked' : self.on_remove_exclude,
				'on_cb_remove_old_backup_toggled' : self.update_remove_old_backups,
				'on_cb_min_free_space_toggled' : self.update_min_free_space,
				'on_cb_per_directory_schedule_toggled' : self.on_cb_per_directory_schedule_toggled,
			}

		self.glade.signal_autoconnect( signals )

		#set current folder
		self.fcb_where = self.glade.get_widget( 'fcb_where' )
		self.fcb_where.set_show_hidden( self.parent.show_hidden_files )
		
		#automatic backup mode store
		self.store_backup_mode = gtk.ListStore( str, int )
		default_automatic_backup_mode_index = 0
		map = self.config.AUTOMATIC_BACKUP_MODES
		self.rev_automatic_backup_modes = {}
		keys = map.keys()
		keys.sort()
		for key in keys:
			self.rev_automatic_backup_modes[ map[key] ] = key
			self.store_backup_mode.append( [ map[key], key ] )

		#per directory schedule
		self.cb_per_directory_schedule = self.glade.get_widget( 'cb_per_directory_schedule' )
		self.lbl_schedule = self.glade.get_widget( 'lbl_schedule' )

		#setup include folders
		self.list_include = self.glade.get_widget( 'list_include' )

		pix_renderer = gtk.CellRendererPixbuf()
		text_renderer = gtk.CellRendererText()

		column = gtk.TreeViewColumn( _('Include folders') )
		column.pack_start( pix_renderer, False )
		column.pack_end( text_renderer, True )
		column.add_attribute( pix_renderer, 'stock-id', 1 )
		column.add_attribute( text_renderer, 'markup', 0 )
		self.list_include.append_column( column )

		column = gtk.TreeViewColumn( _('Schedule') )
		combo_renderer = gtk.CellRendererCombo()
		combo_renderer.set_property( 'editable', True )
		combo_renderer.set_property( 'has-entry', False )
		combo_renderer.set_property( 'model', self.store_backup_mode )
		combo_renderer.set_property( 'text-column', 0 )
		combo_renderer.connect( 'edited', self.on_automatic_backup_mode_changed )
		column.pack_end( combo_renderer, True )
		column.add_attribute( combo_renderer, 'text', 2 )

		self.include_schedule_column = column

		self.store_include = gtk.ListStore( str, str, str, int )
		self.list_include.set_model( self.store_include )

		#setup exclude patterns
		self.list_exclude = self.glade.get_widget( 'list_exclude' )

		pix_renderer = gtk.CellRendererPixbuf()
		text_renderer = gtk.CellRendererText()

		column = gtk.TreeViewColumn( _('Patterns, files or folders') )
		column.pack_start( pix_renderer, False )
		column.pack_end( text_renderer, True )
		column.add_attribute( pix_renderer, 'stock-id', 1 )
		column.add_attribute( text_renderer, 'markup', 0 )
		self.list_exclude.append_column( column )

		self.store_exclude = gtk.ListStore( str, str )
		self.list_exclude.set_model( self.store_exclude )

		#setup automatic backup mode
		self.cb_backup_mode = self.glade.get_widget( 'cb_backup_mode' )
		self.cb_backup_mode.set_model( self.store_backup_mode )

		self.cb_backup_mode.clear()
		renderer = gtk.CellRendererText()
		self.cb_backup_mode.pack_start( renderer, True )
		self.cb_backup_mode.add_attribute( renderer, 'text', 0 )

		#setup remove old backups older than
		self.edit_remove_old_backup_value = self.glade.get_widget( 'edit_remove_old_backup_value' )
		self.cb_remove_old_backup_unit = self.glade.get_widget( 'cb_remove_old_backup_unit' )

		self.store_remove_old_backup_unit = gtk.ListStore( str, int )
		self.cb_remove_old_backup_unit.set_model( self.store_remove_old_backup_unit )

		renderer = gtk.CellRendererText()
		self.cb_remove_old_backup_unit.pack_start( renderer, True )
		self.cb_remove_old_backup_unit.add_attribute( renderer, 'text', 0 )

		self.store_remove_old_backup_unit.clear()
		map = self.config.REMOVE_OLD_BACKUP_UNITS
		keys = map.keys()
		keys.sort()
		for key in keys:
			self.store_remove_old_backup_unit.append( [ map[ key ], key ] )
				
		self.cb_remove_old_backup = self.glade.get_widget( 'cb_remove_old_backup' )

		#setup min free space
		self.edit_min_free_space_value = self.glade.get_widget( 'edit_min_free_space_value' )
		self.cb_min_free_space_unit = self.glade.get_widget( 'cb_min_free_space_unit' )

		self.store_min_free_space_unit = gtk.ListStore( str, int )
		self.cb_min_free_space_unit.set_model( self.store_min_free_space_unit )

		renderer = gtk.CellRendererText()
		self.cb_min_free_space_unit.pack_start( renderer, True )
		self.cb_min_free_space_unit.add_attribute( renderer, 'text', 0 )

		self.store_min_free_space_unit.clear()
		map = self.config.MIN_FREE_SPACE_UNITS
		keys = map.keys()
		keys.sort()
		for key in keys:
			self.store_min_free_space_unit.append( [ map[ key ], key ] )
				
		self.cb_min_free_space = self.glade.get_widget( 'cb_min_free_space' )

		#don't remove named snapshots
		self.cb_dont_remove_named_snapshots = self.glade.get_widget( 'cb_dont_remove_named_snapshots' )
		self.cb_dont_remove_named_snapshots.set_active( self.config.get_dont_remove_named_snapshots() )

		#smart remove
		self.cb_smart_remove = self.glade.get_widget( 'cb_smart_remove' )

		#enable notifications
		self.cb_enable_notifications = self.glade.get_widget( 'cb_enable_notifications' )

		#LOAD FROM CONFIG

		#set current folder
		self.fcb_where.set_filename( self.config.get_snapshots_path() )
		
		#per directory schedule
		self.cb_per_directory_schedule.set_active( self.config.get_per_directory_schedule() )

		#setup include folders
		self.update_per_directory_option()

		self.store_include.clear()
		include_folders = self.config.get_include_folders()
		if len( include_folders ) > 0:
			for include_folder in include_folders:
				self.store_include.append( [include_folder[0], gtk.STOCK_DIRECTORY, self.config.AUTOMATIC_BACKUP_MODES[include_folder[1]], include_folder[1] ] )
		
		#setup exclude patterns
		self.store_exclude.clear()
		exclude_patterns = self.config.get_exclude_patterns()
		if len( exclude_patterns ) > 0:
			for exclude_pattern in exclude_patterns:
				self.store_exclude.append( [exclude_pattern, gtk.STOCK_DELETE] )

		#setup automatic backup mode
		i = 0
		iter = self.store_backup_mode.get_iter_first()
		default_mode = self.config.get_automatic_backup_mode()
		while not iter is None:
			if self.store_backup_mode.get_value( iter, 1 ) == default_mode:
				self.cb_backup_mode.set_active( i )
				break
			iter = self.store_backup_mode.iter_next( iter )
			i = i + 1

		#setup remove old backups older than
		enabled, value, unit = self.config.get_remove_old_snapshots()

		self.edit_remove_old_backup_value.set_value( float( value ) )

		i = 0
		iter = self.store_remove_old_backup_unit.get_iter_first()
		while not iter is None:
			if self.store_remove_old_backup_unit.get_value( iter, 1 ) == unit:
				self.cb_remove_old_backup_unit.set_active( i )
				break
			iter = self.store_remove_old_backup_unit.iter_next( iter )
			i = i + 1
				
		self.cb_remove_old_backup.set_active( enabled )
		self.update_remove_old_backups( self.cb_remove_old_backup )

		#setup min free space
		enabled, value, unit = self.config.get_min_free_space()
		
		self.edit_min_free_space_value.set_value( float(value) )

		i = 0
		iter = self.store_min_free_space_unit.get_iter_first()
		while not iter is None:
			if self.store_min_free_space_unit.get_value( iter, 1 ) == unit:
				self.cb_min_free_space_unit.set_active( i )
				break
			iter = self.store_min_free_space_unit.iter_next( iter )
			i = i + 1

		self.cb_min_free_space.set_active( enabled )
		self.update_min_free_space( self.cb_min_free_space )

		#don't remove named snapshots
		self.cb_dont_remove_named_snapshots.set_active( self.config.get_dont_remove_named_snapshots() )

		#smart remove
		self.cb_smart_remove.set_active( self.config.get_smart_remove() )

		#enable notifications
		self.cb_enable_notifications.set_active( self.config.is_notify_enabled() )

		#run 'nice' from cron
		self.cb_run_nice_from_cron = self.glade.get_widget( 'cb_run_nice_from_cron' )
		self.cb_run_nice_from_cron.set_active( self.config.is_run_nice_from_cron_enabled() )

	def on_automatic_backup_mode_changed( self, renderer, path, new_text ):
		iter = self.store_include.get_iter(path)
		self.store_include.set_value( iter, 2, new_text )
		self.store_include.set_value( iter, 3, self.rev_automatic_backup_modes[new_text] )

	def update_remove_old_backups( self, button ):
		enabled = button.get_active()
		self.edit_remove_old_backup_value.set_sensitive( enabled )
		self.cb_remove_old_backup_unit.set_sensitive( enabled )

	def update_min_free_space( self, button ):
		enabled = button.get_active()
		self.edit_min_free_space_value.set_sensitive( enabled )
		self.cb_min_free_space_unit.set_sensitive( enabled )

	def on_cb_per_directory_schedule_toggled( self, button ):
		self.update_per_directory_option()

	def update_per_directory_option( self ):
		if self.cb_per_directory_schedule.get_active():
			if self.list_include.get_column(1) == None:
				self.list_include.append_column( self.include_schedule_column )
			self.cb_backup_mode.hide()
			self.lbl_schedule.hide()
		else:
			if self.list_include.get_column(1) != None:
				self.list_include.remove_column( self.include_schedule_column )
			self.lbl_schedule.show()
			self.cb_backup_mode.show()

	def run( self ):
		while True:
			if gtk.RESPONSE_OK == self.dialog.run():
				if not self.validate():
					continue
			break
		self.dialog.destroy()

	def on_add_include( self, button ):
		fcd = gtk.FileChooserDialog( _('Include folder'), self.dialog, gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER, (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_OK) )
		fcd.set_show_hidden( self.parent.show_hidden_files  )

		if fcd.run() == gtk.RESPONSE_OK:
			include_folder = fcd.get_filename()

			iter = self.store_include.get_iter_first()
			while not iter is None:
				if self.store_include.get_value( iter, 0 ) == include_folder:
					break
				iter = self.store_include.iter_next( iter )

			if iter is None:
				self.store_include.append( [include_folder, gtk.STOCK_DIRECTORY, self.config.AUTOMATIC_BACKUP_MODES[self.config.NONE], self.config.NONE ] )

		fcd.destroy()

	def on_remove_include( self, button ):
		store, iter = self.list_include.get_selection().get_selected()
		if not iter is None:
			store.remove( iter )

	def add_exclude_( self, pattern ):
		pattern = pattern.strip()
		if len( pattern ) == 0:
			return

		iter = self.store_exclude.get_iter_first()
		while not iter is None:
			if self.store_exclude.get_value( iter, 0 ) == pattern:
				return
			iter = self.store_exclude.iter_next( iter )

		self.store_exclude.append( [pattern, gtk.STOCK_DELETE] )

	def on_add_exclude( self, button ):
		pattern = messagebox.text_input_dialog( self.dialog, self.config, _('Exclude pattern') )
		if pattern is None:
			return

		if pattern.find( ':' ) >= 0:
			messagebox.show_error( self.dialog, self.config, _('Exclude patterns can\'t contain \':\' char !') )
			return
	
		self.add_exclude_( pattern )

	def on_add_exclude_file( self, button ):
		fcd = gtk.FileChooserDialog( _('Exclude file'), self.dialog, gtk.FILE_CHOOSER_ACTION_OPEN, (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_OK) )
		fcd.set_show_hidden( self.parent.show_hidden_files  )

		if fcd.run() == gtk.RESPONSE_OK:
			pattern = fcd.get_filename()
			self.add_exclude_( pattern )

		fcd.destroy()

	def on_add_exclude_folder( self, button ):
		fcd = gtk.FileChooserDialog( _('Exclude folder'), self.dialog, gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER, (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_OK) )
		fcd.set_show_hidden( self.parent.show_hidden_files  )

		if fcd.run() == gtk.RESPONSE_OK:
			pattern = fcd.get_filename()
			self.add_exclude_( pattern )

		fcd.destroy()

	def on_remove_exclude( self, button ):
		store, iter = self.list_exclude.get_selection().get_selected()
		if not iter is None:
			store.remove( iter )

	def on_cancel( self, button ):
		self.dialog.destroy()

	def validate( self ):
		#snapshots path
		snapshots_path = self.fcb_where.get_filename()

		#hack
		if snapshots_path.startswith( '//' ):
			snapshots_path = snapshots_path[ 1 : ]

		#include list 
		include_list = []
		iter = self.store_include.get_iter_first()
		while not iter is None:
			include_list.append( ( self.store_include.get_value( iter, 0 ), self.store_include.get_value( iter, 3 ) ) )
			iter = self.store_include.iter_next( iter )

		#exclude patterns
		exclude_list = []
		iter = self.store_exclude.get_iter_first()
		while not iter is None:
			exclude_list.append( self.store_exclude.get_value( iter, 0 ) )
			iter = self.store_exclude.iter_next( iter )

		#check params
		check_ret_val = self.config.check_take_snapshot_params( snapshots_path, include_list, exclude_list )
		if not check_ret_val is None:
			err_id, err_msg = check_ret_val
			messagebox.show_error( self.dialog, self.config, err_msg )
			return False

		#check if back folder changed
		if len( self.config.get_snapshots_path() ) > 0 and self.config.get_snapshots_path() != snapshots_path:
			if gtk.RESPONSE_YES != messagebox.show_question( self.dialog, self.config, _('Are you sure you want to change snapshots folder ?') ):
				return False 

		#ok let's save to config
		msg = self.config.set_snapshots_path( snapshots_path )
		if not msg is None:
			messagebox.show_error( self.dialog, self.config, msg )
			return False

		self.config.set_include_folders( include_list )
		self.config.set_exclude_patterns( exclude_list )

		#global schedule
		self.config.set_automatic_backup_mode( self.store_backup_mode.get_value( self.cb_backup_mode.get_active_iter(), 1 ) )

		#auto-remove snapshots
		self.config.set_remove_old_snapshots( 
						self.cb_remove_old_backup.get_active(), 
						int( self.edit_remove_old_backup_value.get_value() ),
						self.store_remove_old_backup_unit.get_value( self.cb_remove_old_backup_unit.get_active_iter(), 1 ) )
		self.config.set_min_free_space( 
						self.cb_min_free_space.get_active(), 
						int( self.edit_min_free_space_value.get_value() ),
						self.store_min_free_space_unit.get_value( self.cb_min_free_space_unit.get_active_iter(), 1 ) )
		self.config.set_dont_remove_named_snapshots( self.cb_dont_remove_named_snapshots.get_active() )
		self.config.set_smart_remove( self.cb_smart_remove.get_active() )

		#options
		self.config.set_notify_enabled( self.cb_enable_notifications.get_active() )

		#expert options
		self.config.set_per_directory_schedule( self.cb_per_directory_schedule.get_active() )
		self.config.set_run_nice_from_cron_enabled( self.cb_run_nice_from_cron.get_active() )

		self.config.save()

		msg = self.config.setup_cron()
		if not msg is None:
			messagebox.show_error( self.dialog, self.config, msg )
			return False

		return True
	



#    Back In Time
#    Copyright (C) 2008-2009 Oprea Dan
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os.path
import os
import sys


def get_backintime_path( path ):
	return os.path.join( os.path.dirname( os.path.abspath( os.path.dirname( __file__ ) ) ), path )


def register_backintime_path( path ):
	path = get_backintime_path( path )
	if not path in sys.path:
		sys.path = [path] + sys.path


def read_file( path, default_value = None ):
	ret_val = default_value 

	try:
		file = open( path )
		ret_val = file.read()
		file.close()
	except:
		pass

	return ret_val


def read_file_lines( path, default_value = None ):
	ret_val = default_value 

	try:
		file = open( path )
		ret_val = file.readlines()
		file.close()
	except:
		pass

	return ret_val


def read_command_output( cmd ):
	ret_val = ''

	try:
		pipe = os.popen( cmd )
		ret_val = pipe.read().strip()
		pipe.close() 
	except:
		return ''

	return ret_val


def check_command( cmd ):
	cmd = cmd.strip()

	if len( cmd ) < 1:
		return False

	if os.path.isfile( cmd ):
		return True

	cmd = read_command_output( "which \"%s\"" % cmd )

	if len( cmd ) < 1:
		return False

	if os.path.isfile( cmd ):
		return True

	return False


def make_dirs( path ):
	path = path.rstrip( os.sep )
	if len( path ) <= 0:
		return

	if not os.path.isdir( path ):
		try:
			os.makedirs( path )
		except:
			pass


def process_exists( name ):
	output = read_command_output( "ps -o pid= -C %s" % name )
	return len( output ) > 0


def check_x_server():
	return 0 == os.system( 'xdpyinfo >/dev/null 2>&1' )





#    Back In Time
#    Copyright (C) 2008-2009 Oprea Dan
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os.path
import tools
import types

tools.register_backintime_path( 'common' )
tools.register_backintime_path( 'plugins' )


class Plugin:
	
	def __init__( self ):
		return

	def init( self, snapshots ):
		return True

	def is_gui( self ):
		return False

	def on_process_begins( self ):
		return

	def on_process_ends( self ):
		return

	def on_error( self, code, message ):
		return

	def on_new_snapshot( self, snapshot_id, snapshot_path ):
		return


class PluginManager():
	def __init__( self ):
		self.plugins = []
		self.has_gui_plugins_ = False
		self.plugins_loaded = False

	def load_plugins( self, snapshots, force = False ):
		if self.plugins_loaded and not force:
			return

		self.plugins_loaded = True
		self.plugins = []
		self.has_gui_plugins_ = False
		
		plugins_path = tools.get_backintime_path( 'plugins' )

		for file in os.listdir( plugins_path ):
			if file.endswith( '.py' ) and not file.startswith( '__' ):
				path = os.path.join( plugins_path, file )

				module = __import__( file[ : -3 ] )
				module_dict = module.__dict__
				
 				for key, value in module_dict.items():
					if key.startswith( '__' ):
						continue

					if type(value) is types.ClassType:
						if issubclass( value, Plugin ):
							plugin = value()
							if plugin.init( snapshots ):
								if plugin.is_gui():
									self.has_gui_plugins_ = True
									self.plugins.insert( 0, plugin )
								else:
									self.plugins.append( plugin )

	def has_gui_plugins( self ):
		return self.has_gui_plugins_

	def on_process_begins( self ):
		for plugin in self.plugins:
			plugin.on_process_begins()

	def on_process_ends( self ):
		for plugin in reversed( self.plugins ):
			plugin.on_process_ends()

	def on_error( self, code, message = '' ):
		for plugin in self.plugins:
			plugin.on_error( code, message )

	def on_new_snapshot( self, snapshot_id, snapshot_path ):
		for plugin in self.plugins:
			plugin.on_new_snapshot( snapshot_id, snapshot_path )




#    Back In Time
#    Copyright (C) 2008-2009 Oprea Dan
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os
import os.path
import stat
import sys
import gettext

import config
import logger
import snapshots

_=gettext.gettext


def take_snapshot_now_async():
	app = 'backintime'
	if os.path.isfile( './backintime' ):
		app = './backintime'
	cmd = "%s --backup &" % app
	os.system( cmd )


def take_snapshot( cfg, force = True ):
	logger.openlog()
	snapshots.Snapshots( cfg ).take_snapshot( force )
	logger.closelog()


def print_version( cfg, app_name ):
	print 'Back In Time'
	print 'Version: ' + cfg.VERSION
	print ''
	print 'Back In Time comes with ABSOLUTELY NO WARRANTY.'
	print 'This is free software, and you are welcome to redistribute it'
	print "under certain conditions; type `%s --license\' for details." % app_name
	print ''


def print_help( cfg, app_name ):
	print app_name + ' -b|--backup'
	print '\tTake a snapshot'
	print app_name + ' --backup-job'
	print '\tUsed for cron job: take a snapshot'
	print app_name + ' --snapshots-path'
	print '\tShow the path where is saves the snapshots'
	print app_name + ' --snapshots-list'
	print '\tShow the list of snapshots IDs'
	print app_name + ' --snapshots-list-path'
	print '\tShow the paths to snapshots'
	print app_name + ' --last-snapshot'
	print '\tShow the ID of the last snapshot'
	print app_name + ' --last-snapshot-path'
	print '\tShow the path to the last snapshot'
	print app_name + ' -v|--version'
	print '\tShow version'
	print app_name + ' --license'
	print '\tShow license'
	print app_name + ' -h|--help'
	print '\tShow this help'
	print ''


def start_app( app_name = 'backintime', extra_args = [] ):
	cfg = config.Config()
	print_version( cfg, app_name )

	for arg in sys.argv[ 1 : ]:
		if arg == '--backup' or arg == '-b':
			take_snapshot( cfg, True )
			sys.exit(0)

		if arg == '--backup-job':
			take_snapshot( cfg, False )
			sys.exit(0)

		if arg == '--version' or arg == '-v':
			sys.exit(0)

		if arg == '--license':
			print cfg.get_license()
			sys.exit(0)

		if arg == '--help' or arg == '-h':
			print_help( cfg, app_name )
			sys.exit(0)

		if arg == '--snapshots-path':
			if not cfg.is_configured():
				print "The application is not configured !"
			else:
				print "SnapshotsPath: %s" % cfg.get_snapshots_full_path()
			sys.exit(0)

		if arg == '--snapshots-list':
			if not cfg.is_configured():
				print "The application is not configured !"
			else:
				list = snapshots.Snapshots( cfg ).get_snapshots_list()
				if len( list ) <= 0:
					print "There are no snapshots"
				else:
					for snapshot_id in list:
						print "SnapshotID: %s" % snapshot_id
			sys.exit(0)

		if arg == '--snapshots-list-path':
			if not cfg.is_configured():
				print "The application is not configured !"
			else:
				s = snapshots.Snapshots( cfg )
				list = s.get_snapshots_list()
				if len( list ) <= 0:
					print "There are no snapshots"
				else:
					for snapshot_id in list:
						print "SnapshotPath: %s" % s.get_snapshot_path( snapshot_id )
			sys.exit(0)

		if arg == '--last-snapshot':
			if not cfg.is_configured():
				print "The application is not configured !"
			else:
				list = snapshots.Snapshots( cfg ).get_snapshots_list()
				if len( list ) <= 0:
					print "There are no snapshots"
				else:
					print "SnapshotID: %s" % list[0]
			sys.exit(0)

		if arg == '--last-snapshot-path':
			if not cfg.is_configured():
				print "The application is not configured !"
			else:
				s = snapshots.Snapshots( cfg )
				list = s.get_snapshots_list()
				if len( list ) <= 0:
					print "There are no snapshots"
				else:
					print "SnapshotPath: %s" % s.get_snapshot_path( list[0] )
			sys.exit(0)

		if arg == '--snapshots' or arg == '-s':
			continue

		if arg == '--gnome' or arg == '--kde4' or arg == '--kde3':
			continue

		if arg[0] == '-':
			if not arg[0] in extra_args:
				print "Ignore option: %s" % arg
			continue

	return cfg


if __name__ == '__main__':
	start_app()

